# ***************************************************************************
# *   (c) 2009 Yorik van Havre <yorik@uncreated.net>                        *
# *   (c) 2010 Ken Cline <cline@frii.com>                                   *
# *   (c) 2020 Eliud Cabrera Castillo <e.cabrera-castillo@tum.de>           *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful,            *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with FreeCAD; if not, write to the Free Software        *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""Provides tools for creating regular polygons with the Draft Workbench.

Minimum number of sides is three (equilateral triangles).
"""
## @package gui_polygons
# \ingroup DRAFT
# \brief Provides tools for creating regular polygons with the Draft Workbench.

from PySide.QtCore import QT_TRANSLATE_NOOP

import FreeCAD as App
import FreeCADGui as Gui
import DraftVecUtils
import draftutils.utils as utils
import draftguitools.gui_base_original as gui_base_original
import draftguitools.gui_tool_utils as gui_tool_utils
import draftguitools.gui_trackers as trackers
from draftutils.messages import _msg
from draftutils.translate import translate


class Polygon(gui_base_original.Creator):
    """Gui command for the Polygon tool."""

    def GetResources(self):
        """Set icon, menu and tooltip."""
        _tip = ("Creates a regular polygon (triangle, square, "
                "pentagon, ...), by defining the number of sides "
                "and the circumscribed radius.\n"
                "CTRL to snap, SHIFT to constrain")

        return {'Pixmap': 'Draft_Polygon',
                'Accel': "P, G",
                'MenuText': QT_TRANSLATE_NOOP("Draft_Polygon", "Polygon"),
                'ToolTip': QT_TRANSLATE_NOOP("Draft_Polygon", _tip)}

    def Activated(self):
        """Execute when the command is called."""
        name = translate("draft", "Polygon")
        super(Polygon, self).Activated(name)
        if self.ui:
            self.step = 0
            self.center = None
            self.rad = None
            self.tangents = []
            self.tanpoints = []
            self.ui.pointUi(name)
            self.ui.extUi()
            self.ui.numFaces.show()
            self.ui.numFacesLabel.show()
            self.altdown = False
            self.ui.sourceCmd = self
            self.arctrack = trackers.arcTracker()
            self.call = self.view.addEventCallback("SoEvent", self.action)
            _msg(translate("draft", "Pick center point"))

    def finish(self, closed=False, cont=False):
        """Terminate the operation."""
        super(Polygon, self).finish(self)
        if self.ui:
            self.arctrack.finalize()
            self.doc.recompute()
            if self.ui.continueMode:
                self.Activated()

    def action(self, arg):
        """Handle the 3D scene events.

        This is installed as an EventCallback in the Inventor view.

        Parameters
        ----------
        arg: dict
            Dictionary with strings that indicates the type of event received
            from the 3D view.
        """
        import DraftGeomUtils

        if arg["Type"] == "SoKeyboardEvent":
            if arg["Key"] == "ESCAPE":
                self.finish()
        elif arg["Type"] == "SoLocation2Event":  # mouse movement detection
            self.point, ctrlPoint, info = gui_tool_utils.getPoint(self, arg)

            # this is to make sure radius is what you see on screen
            if self.center and DraftVecUtils.dist(self.point, self.center) > 0:
                viewdelta = DraftVecUtils.project(self.point.sub(self.center),
                                                  App.DraftWorkingPlane.axis)
                if not DraftVecUtils.isNull(viewdelta):
                    self.point = self.point.add(viewdelta.negative())
            if self.step == 0:  # choose center
                if gui_tool_utils.hasMod(arg, gui_tool_utils.MODALT):
                    if not self.altdown:
                        self.altdown = True
                        self.ui.switchUi(True)
                else:
                    if self.altdown:
                        self.altdown = False
                        self.ui.switchUi(False)
            else:  # choose radius
                if len(self.tangents) == 2:
                    cir = DraftGeomUtils.circleFrom2tan1pt(self.tangents[0],
                                                           self.tangents[1],
                                                           self.point)
                    _c = DraftGeomUtils.findClosestCircle(self.point, cir)
                    self.center = _c.Center
                    self.arctrack.setCenter(self.center)
                elif self.tangents and self.tanpoints:
                    cir = DraftGeomUtils.circleFrom1tan2pt(self.tangents[0],
                                                           self.tanpoints[0],
                                                           self.point)
                    _c = DraftGeomUtils.findClosestCircle(self.point, cir)
                    self.center = _c.Center
                    self.arctrack.setCenter(self.center)
                if gui_tool_utils.hasMod(arg, gui_tool_utils.MODALT):
                    if not self.altdown:
                        self.altdown = True
                    snapped = self.view.getObjectInfo((arg["Position"][0],
                                                       arg["Position"][1]))
                    if snapped:
                        ob = self.doc.getObject(snapped['Object'])
                        num = int(snapped['Component'].lstrip('Edge')) - 1
                        ed = ob.Shape.Edges[num]
                        if len(self.tangents) == 2:
                            cir = DraftGeomUtils.circleFrom3tan(self.tangents[0],
                                                                self.tangents[1],
                                                                ed)
                            cl = DraftGeomUtils.findClosestCircle(self.point, cir)
                            self.center = cl.Center
                            self.rad = cl.Radius
                            self.arctrack.setCenter(self.center)
                        else:
                            self.rad = self.center.add(DraftGeomUtils.findDistance(self.center,ed).sub(self.center)).Length
                    else:
                        self.rad = DraftVecUtils.dist(self.point, self.center)
                else:
                    if self.altdown:
                        self.altdown = False
                    self.rad = DraftVecUtils.dist(self.point, self.center)
                self.ui.setRadiusValue(self.rad, 'Length')
                self.arctrack.setRadius(self.rad)

            gui_tool_utils.redraw3DView()

        elif (arg["Type"] == "SoMouseButtonEvent"
              and arg["State"] == "DOWN"
              and arg["Button"] == "BUTTON1"):  # mouse click
            if self.point:
                if self.step == 0:  # choose center
                    if (not self.node) and (not self.support):
                        gui_tool_utils.getSupport(arg)
                        (self.point,
                         ctrlPoint, info) = gui_tool_utils.getPoint(self, arg)
                    if gui_tool_utils.hasMod(arg, gui_tool_utils.MODALT):
                        snapped = self.view.getObjectInfo((arg["Position"][0],
                                                           arg["Position"][1]))
                        if snapped:
                            ob = self.doc.getObject(snapped['Object'])
                            num = int(snapped['Component'].lstrip('Edge')) - 1
                            ed = ob.Shape.Edges[num]
                            self.tangents.append(ed)
                            if len(self.tangents) == 2:
                                self.arctrack.on()
                                self.ui.radiusUi()
                                self.step = 1
                                _msg(translate("draft", "Pick radius"))
                    else:
                        if len(self.tangents) == 1:
                            self.tanpoints.append(self.point)
                        else:
                            self.center = self.point
                            self.node = [self.point]
                            self.arctrack.setCenter(self.center)
                        self.arctrack.on()
                        self.ui.radiusUi()
                        self.step = 1
                        _msg(translate("draft", "Pick radius"))
                        if self.planetrack:
                            self.planetrack.set(self.point)
                elif self.step == 1:  # choose radius
                    self.drawPolygon()

    def drawPolygon(self):
        """Draw the actual object."""
        rot, sup, pts, fil = self.getStrings()
        Gui.addModule("Draft")
        if utils.getParam("UsePartPrimitives", False):
            # Insert a Part::Primitive object
            Gui.addModule("Part")
            _cmd = 'FreeCAD.ActiveDocument.'
            _cmd += 'addObject("Part::RegularPolygon","RegularPolygon")'
            _cmd_list = ['pl = FreeCAD.Placement()',
                         'pl.Rotation.Q = ' + rot,
                         'pl.Base = ' + DraftVecUtils.toString(self.center),
                         'pol = ' + _cmd,
                         'pol.Polygon = ' + str(self.ui.numFaces.value()),
                         'pol.Circumradius = ' + str(self.rad),
                         'pol.Placement = pl',
                         'Draft.autogroup(pol)',
                         'FreeCAD.ActiveDocument.recompute()']
            self.commit(translate("draft", "Create Polygon (Part)"),
                        _cmd_list)
        else:
            # Insert a Draft polygon
            _cmd = 'Draft.makePolygon'
            _cmd += '('
            _cmd += str(self.ui.numFaces.value()) + ', '
            _cmd += 'radius=' + str(self.rad) + ', '
            _cmd += 'inscribed=True, '
            _cmd += 'placement=pl, '
            _cmd += 'face=' + fil + ', '
            _cmd += 'support=' + sup
            _cmd += ')'
            _cmd_list = ['pl = FreeCAD.Placement()',
                         'pl.Rotation.Q = ' + rot,
                         'pl.Base = ' + DraftVecUtils.toString(self.center),
                         'pol = ' + _cmd,
                         'Draft.autogroup(pol)',
                         'FreeCAD.ActiveDocument.recompute()']
            self.commit(translate("draft", "Create Polygon"),
                        _cmd_list)
        self.finish(cont=True)

    def numericInput(self, numx, numy, numz):
        """Validate the entry fields in the user interface.

        This function is called by the toolbar or taskpanel interface
        when valid x, y, and z have been entered in the input fields.
        """
        self.center = App.Vector(numx, numy, numz)
        self.node = [self.center]
        self.arctrack.setCenter(self.center)
        self.arctrack.on()
        self.ui.radiusUi()
        self.step = 1
        self.ui.radiusValue.setFocus()
        _msg(translate("draft", "Pick radius"))

    def numericRadius(self, rad):
        """Validate the entry radius in the user interface.

        This function is called by the toolbar or taskpanel interface
        when a valid radius has been entered in the input field.
        """
        import DraftGeomUtils

        self.rad = rad
        if len(self.tangents) == 2:
            cir = DraftGeomUtils.circleFrom2tan1rad(self.tangents[0],
                                                    self.tangents[1],
                                                    rad)
            if self.center:
                _c = DraftGeomUtils.findClosestCircle(self.center, cir)
                self.center = _c.Center
            else:
                self.center = cir[-1].Center
        elif self.tangents and self.tanpoints:
            cir = DraftGeomUtils.circleFrom1tan1pt1rad(self.tangents[0],
                                                       self.tanpoints[0],
                                                       rad)
            if self.center:
                _c = DraftGeomUtils.findClosestCircle(self.center, cir)
                self.center = _c.Center
            else:
                self.center = cir[-1].Center
        self.drawPolygon()


Gui.addCommand('Draft_Polygon', Polygon())



# ***************************************************************************
# *   (c) 2009, 2010 Yorik van Havre <yorik@uncreated.net>                  *
# *   (c) 2009, 2010 Ken Cline <cline@frii.com>                             *
# *   (c) 2020 Eliud Cabrera Castillo <e.cabrera-castillo@tum.de>           *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful,            *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with FreeCAD; if not, write to the Free Software        *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""Provides tools to modify Draft dimensions.

For example, a tool to flip the direction of the text in the dimension
as the normal is sometimes not correctly calculated automatically.
"""
## @package gui_dimension_ops
# \ingroup DRAFT
# \brief Provides tools to modify Draft dimensions.

from PySide.QtCore import QT_TRANSLATE_NOOP

import FreeCADGui as Gui
import draftutils.utils as utils
import draftguitools.gui_base as gui_base
from draftutils.translate import _tr


class FlipDimension(gui_base.GuiCommandNeedsSelection):
    """The Draft FlipDimension command definition.

    Flip the normal direction of the selected dimensions.

    It inherits `GuiCommandNeedsSelection` to set up the document
    and other behavior. See this class for more information.
    """

    def __init__(self):
        super(Draft_FlipDimension, self).__init__(name=_tr("Flip dimension"))

    def GetResources(self):
        """Set icon, menu and tooltip."""
        _tip = ("Flip the normal direction of the selected dimensions "
                "(linear, radial, angular).\n"
                "If other objects are selected they are ignored.")

        return {'Pixmap': 'Draft_FlipDimension',
                'MenuText': QT_TRANSLATE_NOOP("Draft_FlipDimension",
                                              "Flip dimension"),
                'ToolTip': QT_TRANSLATE_NOOP("Draft_FlipDimension",
                                             _tip)}

    def Activated(self):
        """Execute when the command is called."""
        super(Draft_FlipDimension, self).Activated()

        for o in Gui.Selection.getSelection():
            if utils.get_type(o) in ("Dimension", "AngularDimension"):
                self.doc.openTransaction("Flip dimension")
                _cmd = "App.activeDocument()." + o.Name + ".Normal"
                _cmd += " = "
                _cmd += "App.activeDocument()." + o.Name + ".Normal.negative()"
                Gui.doCommand(_cmd)
                self.doc.commitTransaction()
                self.doc.recompute()


Draft_FlipDimension = FlipDimension
Gui.addCommand('Draft_FlipDimension', FlipDimension())



# ***************************************************************************
# *   Copyright (c) 2015 Przemo Firszt <przemo@firszt.eu>                   *
# *   Copyright (c) 2016 Bernd Hahnebach <bernd@bimstatik.org>              *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FemToolsCcx"
__author__ = "Przemo Firszt, Bernd Hahnebach"
__url__ = "http://www.freecadweb.org"

## \addtogroup FEM
#  @{

import os
import sys
import subprocess

import FreeCAD

from femtools import femutils
from femtools import membertools

from PySide import QtCore  # there might be a special reason this is not guarded ?!?
if FreeCAD.GuiUp:
    from PySide import QtGui
    import FemGui


class FemToolsCcx(QtCore.QRunnable, QtCore.QObject):
    """

    Attributes
    ----------
    analysis : Fem::FemAnalysis
        FEM group analysis object
        has to be present, will be set in __init__
    solver : Fem::FemSolverObjectPython
        FEM solver object
        has to be present, will be set in __init__
    base_name : str
        name of .inp/.frd file (without extension)
        It is used to construct .inp file path that is passed to CalculiX ccx
    ccx_binary : str
    working_dir : str
    results_present : bool
        indicating if there are calculation results ready for us
    members : class femtools/membertools/AnalysisMember
        contains references to all analysis member except solvers and mesh
        Updated with update_objects
    """

    finished = QtCore.Signal(int)

    def __init__(self, analysis=None, solver=None, test_mode=False):
        """The constructor

        Parameters
        ----------
        analysis : Fem::FemAnalysis, optional
            analysis group as a container for all  objects needed for the analysis
        solver : Fem::FemSolverObjectPython, optional
            solver object to be used for this solve
        test_mode : bool, optional
            mainly used in unit tests
        """

        QtCore.QRunnable.__init__(self)
        QtCore.QObject.__init__(self)

        self.analysis = None
        self.solver = None

        if analysis:
            self.analysis = analysis
            if solver:
                # analysis and solver given
                self.solver = solver
            else:
                # analysis given, search for the solver
                self.find_solver()
                if not self.solver:
                    raise Exception("FEM: No solver found!")
        else:
            if solver:
                # solver given, search for the analysis
                self.solver = solver
                self.find_solver_analysis()
                if not self.analysis:
                    raise Exception(
                        "FEM: The solver was given as parameter, "
                        "but no analysis for this solver was found!"
                    )
            else:
                # neither analysis nor solver given, search both
                self.find_analysis()
                if not self.analysis:
                    raise Exception(
                        "FEM: No solver was given and either no active analysis "
                        "or no analysis at all or more than one analysis found!"
                    )
                self.find_solver()
                if not self.solver:
                    raise Exception("FEM: No solver found!")

        if self.analysis.Document is not self.solver.Document:
            raise Exception(
                "FEM: The analysis and solver are not in the same document!"
            )
        if self.solver not in self.analysis.Group:
            raise Exception(
                "FEM: The solver is not part of the analysis Group!"
            )

        # print(self.solver)
        # print(self.analysis)
        if self.analysis and self.solver:
            self.working_dir = ""
            self.ccx_binary = ""
            self.base_name = ""
            self.results_present = False
            if test_mode:
                self.test_mode = True
                self.ccx_binary_present = True
            else:
                self.test_mode = False
                self.ccx_binary_present = False
            self.result_object = None
        else:
            raise Exception(
                "FEM: Something went wrong, "
                "the exception should have been raised earlier!"
            )

    def purge_results(self):
        """Remove all result objects and result meshes from an analysis group
        """
        from femresult.resulttools import purge_results as pr
        pr(self.analysis)

    def reset_mesh_purge_results_checked(self):
        """Reset mesh color, deformation and removes all result objects
        if preferences to keep them is not set.
        """
        self.fem_prefs = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Fem/General")
        keep_results_on_rerun = self.fem_prefs.GetBool("KeepResultsOnReRun", False)
        if not keep_results_on_rerun:
            self.purge_results()

    def reset_all(self):
        """Reset mesh color, deformation and removes all result objects
        """
        self.purge_results()

    def _get_several_member(self, obj_type):
        return membertools.get_several_member(self.analysis, obj_type)

    def find_analysis(self):
        if FreeCAD.GuiUp:
            self.analysis = FemGui.getActiveAnalysis()
        if self.analysis:
            return
        found_analysis = False
        # search in the active document
        for m in FreeCAD.activeDocument().Objects:
            if femutils.is_of_type(m, "Fem::FemAnalysis"):
                if not found_analysis:
                    self.analysis = m
                    found_analysis = True
                else:
                    self.analysis = None  # more than one analysis
        if self.analysis:
            if FreeCAD.GuiUp:
                FemGui.setActiveAnalysis(self.analysis)

    def find_solver_analysis(self):
        """ get the analysis group the solver belongs to
        """
        if self.solver.getParentGroup():
            obj = self.solver.getParentGroup()
            if femutils.is_of_type(obj, "Fem::FemAnalysis"):
                self.analysis = obj
                if FreeCAD.GuiUp:
                    FemGui.setActiveAnalysis(self.analysis)

    def find_solver(self):
        found_solver_for_use = False
        for m in self.analysis.Group:
            if femutils.is_of_type(m, "Fem::FemSolverCalculixCcxTools"):
                # we are going to explicitly check for the ccx tools solver type only,
                # thus it is possible to have lots of framework solvers inside the analysis anyway
                # for some methods no solver is needed (purge_results) --> solver could be none
                # analysis has one solver and no solver was set --> use the one solver
                # analysis has more than one solver and no solver was set --> use solver none
                # analysis has no solver --> use solver none
                if not found_solver_for_use:
                    # no solver was found before
                    self.solver = m
                    found_solver_for_use = True
                else:
                    # another solver was found --> We have more than one solver
                    # we do not know which one to use, so we use none !
                    self.solver = None
                    FreeCAD.Console.PrintLog(
                        "FEM: More than one solver in the analysis "
                        "and no solver given to analyze. "
                        "No solver is set!\n"
                    )

    def update_objects(self):
        ## @var mesh
        #  mesh of the analysis. Used to generate .inp file and to show results
        self.mesh = None
        mesh, message = membertools.get_mesh_to_solve(self.analysis)
        if mesh is not None:
            self.mesh = mesh
        else:
            if FreeCAD.GuiUp:
                QtGui.QMessageBox.critical(None, "Missing prerequisite", message)
            raise Exception(message + "\n")

        ## @var members
        # members of the analysis. All except solvers and the mesh
        self.member = membertools.AnalysisMember(self.analysis)

    def check_prerequisites(self):
        FreeCAD.Console.PrintMessage("Check prerequisites.\n")
        message = ""
        # analysis
        if not self.analysis:
            message += "No active Analysis\n"
        # solver
        if not self.solver:
            message += "No solver object defined in the analysis\n"
        if not self.working_dir:
            message += "Working directory not set\n"
        if not (os.path.isdir(self.working_dir)):
            message += (
                "Working directory \'{}\' doesn't exist."
                .format(self.working_dir)
            )
        from femtools.checksanalysis import check_analysismember
        message += check_analysismember(
            self.analysis,
            self.solver,
            self.mesh,
            self.member
        )
        return message

    def set_base_name(self, base_name=None):
        """
        Set base_name

        Parameters
        ----------
        base_name : str, optional
            base_name base name of .inp/.frd file (without extension).
            It is used to construct .inp file path that is passed to CalculiX ccx
        """
        if base_name is None:
            self.base_name = ""
        else:
            self.base_name = base_name
        # Update inp file name
        self.set_inp_file_name()

    def set_inp_file_name(self, inp_file_name=None):
        """
        Set inp file name. Normally inp file name is set by write_inp_file.
        That name is also used to determine location and name of frd result file.

        Parameters
        ----------
        inp_file_name : str, optional
            input file name path
        """
        if inp_file_name is not None:
            self.inp_file_name = inp_file_name
        else:
            self.inp_file_name = os.path.join(self.working_dir, (self.base_name + ".inp"))

    def setup_working_dir(self, param_working_dir=None, create=False):
        """Set working dir for solver execution.

        Parameters
        ----------
        param_working_dir :  str, optional
            directory to be used for writing
        create : bool, optional
            Should the working directory be created if it does not exist
        """
        self.working_dir = ""
        # try to use given working dir or overwrite with solver working dir
        fem_general_prefs = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Fem/General")
        if param_working_dir is not None:
            self.working_dir = param_working_dir
            if femutils.check_working_dir(self.working_dir) is not True:
                if create is True:
                    FreeCAD.Console.PrintMessage(
                        "Dir given as parameter \'{}\' doesn't exist.\n".format(self.working_dir)
                    )
                else:
                    FreeCAD.Console.PrintError(
                        "Dir given as parameter \'{}\' doesn't exist "
                        "and create parameter is set to False.\n"
                        .format(self.working_dir)
                    )
                    self.working_dir = femutils.get_pref_working_dir(self.solver)
                    FreeCAD.Console.PrintMessage(
                        "Dir \'{}\' will be used instead.\n"
                        .format(self.working_dir)
                    )
        elif fem_general_prefs.GetBool("OverwriteSolverWorkingDirectory", True) is False:
            self.working_dir = self.solver.WorkingDir
            if femutils.check_working_dir(self.working_dir) is not True:
                if self.working_dir == '':
                    FreeCAD.Console.PrintError(
                        "Working Dir is set to be used from solver object "
                        "but Dir from solver object \'{}\' is empty.\n"
                        .format(self.working_dir)
                    )
                else:
                    FreeCAD.Console.PrintError(
                        "Dir from solver object \'{}\' doesn't exist.\n"
                        .format(self.working_dir)
                    )
                self.working_dir = femutils.get_pref_working_dir(self.solver)
                FreeCAD.Console.PrintMessage(
                    "Dir \'{}\' will be used instead.\n"
                    .format(self.working_dir)
                )
        else:
            self.working_dir = femutils.get_pref_working_dir(self.solver)

        # check working_dir exist, if not use a tmp dir and inform the user
        if femutils.check_working_dir(self.working_dir) is not True:
            FreeCAD.Console.PrintError(
                "Dir \'{}\' doesn't exist or cannot be created.\n"
                .format(self.working_dir)
            )
            self.working_dir = femutils.get_temp_dir(self.solver)
            FreeCAD.Console.PrintMessage(
                "Dir \'{}\' will be used instead.\n"
                .format(self.working_dir)
            )

        # Update inp file name
        self.set_inp_file_name()

    def write_inp_file(self):
        import femsolver.calculix.writer as iw
        self.inp_file_name = ""
        try:
            inp_writer = iw.FemInputWriterCcx(
                self.analysis,
                self.solver,
                self.mesh,
                self.member,
                self.working_dir
            )
            self.inp_file_name = inp_writer.write_calculix_input_file()
        except:
            FreeCAD.Console.PrintError(
                "Unexpected error when writing CalculiX input file: {}\n"
                .format(sys.exc_info()[0])
            )
            raise

    def setup_ccx(self, ccx_binary=None, ccx_binary_sig="CalculiX"):
        """Set Calculix binary path and validate its execution.

        Parameters
        ----------
        ccx_binary : str, optional
            It defaults to `None`. The path to the `ccx` binary. If it is `None`,
            the path is guessed.
        ccx_binary_sig : str, optional
            Defaults to 'CalculiX'. Expected output from `ccx` when run empty.

        Raises
        ------
        Exception
        """
        error_title = "No CalculiX binary ccx"
        error_message = ""
        from platform import system
        ccx_std_location = FreeCAD.ParamGet(
            "User parameter:BaseApp/Preferences/Mod/Fem/Ccx"
        ).GetBool("UseStandardCcxLocation", True)
        if ccx_std_location:
            if system() == "Windows":
                ccx_path = FreeCAD.getHomePath() + "bin/ccx.exe"
                FreeCAD.ParamGet(
                    "User parameter:BaseApp/Preferences/Mod/Fem/Ccx"
                ).SetString("ccxBinaryPath", ccx_path)
                self.ccx_binary = ccx_path
            elif system() in ("Linux", "Darwin"):
                p1 = subprocess.Popen(["which", "ccx"], stdout=subprocess.PIPE)
                if p1.wait() == 0:
                    if sys.version_info.major >= 3:
                        ccx_path = p1.stdout.read().decode("utf8").split("\n")[0]
                    else:
                        ccx_path = p1.stdout.read().split("\n")[0]
                elif p1.wait() == 1:
                    error_message = (
                        "FEM: CalculiX binary ccx not found in "
                        "standard system binary path. "
                        "Please install ccx or set path to binary "
                        "in FEM preferences tab CalculiX.\n"
                    )
                    if FreeCAD.GuiUp:
                        QtGui.QMessageBox.critical(None, error_title, error_message)
                    raise Exception(error_message)
                self.ccx_binary = ccx_path
        else:
            if not ccx_binary:
                self.ccx_prefs = FreeCAD.ParamGet(
                    "User parameter:BaseApp/Preferences/Mod/Fem/Ccx"
                )
                ccx_binary = self.ccx_prefs.GetString("ccxBinaryPath", "")
                if not ccx_binary:
                    FreeCAD.ParamGet(
                        "User parameter:BaseApp/Preferences/Mod/Fem/Ccx"
                    ).SetBool("UseStandardCcxLocation", True)
                    error_message = (
                        "FEM: CalculiX binary ccx path not set at all. "
                        "The use of standard path was activated in "
                        "FEM preferences tab CalculiX. Please try again!\n"
                    )
                    if FreeCAD.GuiUp:
                        QtGui.QMessageBox.critical(None, error_title, error_message)
                    raise Exception(error_message)
            self.ccx_binary = ccx_binary

        startup_info = None
        if system() == "Windows":
            # Windows workaround to avoid blinking terminal window
            startup_info = subprocess.STARTUPINFO()
            startup_info.dwFlags = subprocess.STARTF_USESHOWWINDOW
        ccx_stdout = None
        ccx_stderr = None
        try:
            p = subprocess.Popen(
                [self.ccx_binary],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                shell=False,
                startupinfo=startup_info
            )
            ccx_stdout, ccx_stderr = p.communicate()
            if ccx_binary_sig in str(ccx_stdout):
                self.ccx_binary_present = True
            else:
                raise Exception("FEM: wrong ccx binary")
                # since we raise an exception the try will fail and
                # the exception later with the error popup will be raised
                # TODO: I'm still able to break it.
                # If user doesn't give a file but a path without a file or
                # a file which is not a binary no exception at all is raised.
        except OSError as e:
            FreeCAD.Console.PrintError("{}\n".format(e))
            if e.errno == 2:
                error_message = (
                    "FEM: CalculiX binary ccx \'{}\' not found. "
                    "Please set the CalculiX binary ccx path in "
                    "FEM preferences tab CalculiX.\n"
                    .format(ccx_binary)
                )
                if FreeCAD.GuiUp:
                    QtGui.QMessageBox.critical(None, error_title, error_message)
                raise Exception(error_message)
        except Exception as e:
            FreeCAD.Console.PrintError("{}\n".format(e))
            error_message = (
                "FEM: CalculiX ccx \'{}\' output \'{}\' doesn't "
                "contain expected phrase \'{}\'. "
                "There are some problems when running the ccx binary. "
                "Check if ccx runs standalone without FreeCAD.\n"
                .format(ccx_binary, ccx_stdout, ccx_binary_sig)
            )
            if FreeCAD.GuiUp:
                QtGui.QMessageBox.critical(None, error_title, error_message)
            raise Exception(error_message)

    def start_ccx(self):
        import multiprocessing
        self.ccx_stdout = ""
        self.ccx_stderr = ""
        ont_backup = os.environ.get("OMP_NUM_THREADS")
        self.ccx_prefs = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Fem/Ccx")
        # If number of CPU's specified
        num_cpu_pref = self.ccx_prefs.GetInt("AnalysisNumCPUs", 1)
        if not ont_backup:
            ont_backup = str(num_cpu_pref)
        if num_cpu_pref > 1:
            # If user picked a number use that instead
            _env = os.putenv("OMP_NUM_THREADS", str(num_cpu_pref))
        else:
            _env = os.putenv("OMP_NUM_THREADS", str(multiprocessing.cpu_count()))
        # change cwd because ccx may crash if directory has no write permission
        # there is also a limit of the length of file names so jump to the document directory
        cwd = QtCore.QDir.currentPath()
        f = QtCore.QFileInfo(self.inp_file_name)
        QtCore.QDir.setCurrent(f.path())
        p = subprocess.Popen(
            [self.ccx_binary, "-i ", f.baseName()],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            shell=False,
            env=_env
        )
        self.ccx_stdout, self.ccx_stderr = p.communicate()
        if sys.version_info.major >= 3:
            self.ccx_stdout = self.ccx_stdout.decode()
            self.ccx_stderr = self.ccx_stderr.decode()
        os.putenv("OMP_NUM_THREADS", ont_backup)
        QtCore.QDir.setCurrent(cwd)
        return p.returncode

    def get_ccx_version(self):
        self.setup_ccx()
        import re
        from platform import system
        startup_info = None
        if system() == "Windows":
            # Windows workaround to avoid blinking terminal window
            startup_info = subprocess.STARTUPINFO()
            startup_info.dwFlags = subprocess.STARTF_USESHOWWINDOW
        ccx_stdout = None
        ccx_stderr = None
        # Now extract the version number
        p = subprocess.Popen(
            [self.ccx_binary, "-v"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            shell=False,
            startupinfo=startup_info
        )
        ccx_stdout, ccx_stderr = p.communicate()
        if sys.version_info.major >= 3:
            ccx_stdout = ccx_stdout.decode()
            # ccx_stderr = ccx_stderr.decode()
        m = re.search(r"(\d+).(\d+)", ccx_stdout)
        return (int(m.group(1)), int(m.group(2)))

    def ccx_run(self):
        FreeCAD.Console.PrintMessage("Run CalculiX ...\n")
        if self.test_mode:
            FreeCAD.Console.PrintError("CalculiX can not be run if test_mode is True.\n")
            return
        self.setup_ccx()
        if self.ccx_binary_present is False:
            error_message = (
                "FEM: CalculiX binary ccx \'{}\' not found. "
                "Please set the CalculiX binary ccx path in FEM preferences tab CalculiX.\n"
                .format(self.ccx_binary)
            )
            if FreeCAD.GuiUp:
                QtGui.QMessageBox.critical(None, "No CalculiX binary ccx", error_message)
            raise Exception(error_message)
        progress_bar = FreeCAD.Base.ProgressIndicator()
        progress_bar.start("Everything seams fine. CalculiX ccx will be executed ...", 0)
        ret_code = 0
        ret_code = self.start_ccx()
        self.finished.emit(ret_code)
        progress_bar.stop()
        if ret_code or self.ccx_stderr:
            if ret_code == 201 and self.solver.AnalysisType == "check":
                FreeCAD.Console.PrintMessage(
                    "It seams we run into NOANALYSIS problem, "
                    "thus workaround for wrong exit code for *NOANALYSIS check "
                    "and set ret_code to 0.\n"
                )
                # https://forum.freecadweb.org/viewtopic.php?f=18&t=31303&start=10#p260743
                ret_code = 0
            else:
                FreeCAD.Console.PrintError("CalculiX failed with exit code {}\n".format(ret_code))
                FreeCAD.Console.PrintMessage("--------start of stderr-------\n")
                FreeCAD.Console.PrintMessage(self.ccx_stderr)
                FreeCAD.Console.PrintMessage("--------end of stderr---------\n")
                FreeCAD.Console.PrintMessage("--------start of stdout-------\n")
                FreeCAD.Console.PrintMessage(self.ccx_stdout)
                FreeCAD.Console.PrintMessage("\n--------end of stdout---------\n")
                FreeCAD.Console.PrintMessage("--------start problems---------\n")
                self.has_no_material_assigned()
                self.has_nonpositive_jacobians()
                FreeCAD.Console.PrintMessage("\n--------end problems---------\n")
        else:
            FreeCAD.Console.PrintMessage("CalculiX finished without error.\n")
        return ret_code

    def run(self):
        self.update_objects()
        self.setup_working_dir()
        message = self.check_prerequisites()
        if message:
            error_message = (
                "CalculiX was not started due to missing prerequisites:\n{}\n"
                .format(message)
            )
            FreeCAD.Console.PrintError(error_message)
            if FreeCAD.GuiUp:
                QtGui.QMessageBox.critical(
                    None,
                    "Missing prerequisite",
                    error_message
                )
            return False
        else:
            self.write_inp_file()
            if self.inp_file_name == "":
                error_message = "Error on writing CalculiX input file.\n"
                FreeCAD.Console.PrintError(error_message)
                if FreeCAD.GuiUp:
                    QtGui.QMessageBox.critical(
                        None,
                        "Error",
                        error_message
                    )
                return False
            else:
                FreeCAD.Console.PrintMessage(
                    "Writing CalculiX input file completed.\n"
                )
                ret_code = self.ccx_run()
                if ret_code != 0:
                    error_message = (
                        "CalculiX finished with error {}.\n"
                        .format(ret_code)
                    )
                    FreeCAD.Console.PrintError(error_message)
                    if FreeCAD.GuiUp:
                        QtGui.QMessageBox.critical(
                            None,
                            "Error",
                            error_message
                        )
                    return False
                else:
                    self.load_results()
                    # TODO: output an error message if there where problems reading the results
        return True

    def has_no_material_assigned(self):
        if " *ERROR in calinput: no material was assigned" in self.ccx_stdout:
            without_material_elements = []
            without_material_elemnodes = []
            for line in self.ccx_stdout.splitlines():
                if "to element" in line:
                    # print(line)
                    # print(line.split())
                    non_mat_ele = int(line.split()[2])
                    # print(non_mat_ele)
                    if non_mat_ele not in without_material_elements:
                        without_material_elements.append(non_mat_ele)
            for e in without_material_elements:
                for n in self.mesh.FemMesh.getElementNodes(e):
                    without_material_elemnodes.append(n)
            without_material_elements = sorted(without_material_elements)
            without_material_elemnodes = sorted(without_material_elemnodes)
            command_for_withoutmatnodes = (
                "without_material_elemnodes = {}"
                .format(without_material_elemnodes)
            )
            command_to_highlight = (
                "Gui.ActiveDocument.{}.HighlightedNodes = without_material_elemnodes"
                .format(self.mesh.Name)
            )
            # some output for the user
            FreeCAD.Console.PrintError(
                "\n\nCalculiX returned an error due to elements without materials.\n"
            )
            FreeCAD.Console.PrintMessage(
                "without_material_elements = {}\n"
                .format(without_material_elements)
            )
            FreeCAD.Console.PrintMessage(command_for_withoutmatnodes + "\n")
            if FreeCAD.GuiUp:
                import FreeCADGui
                # with this the list without_material_elemnodes
                # will be available for further user interaction
                FreeCADGui.doCommand(command_for_withoutmatnodes)
                FreeCAD.Console.PrintMessage("\n")
                FreeCADGui.doCommand(command_to_highlight)
            FreeCAD.Console.PrintMessage(
                "\nFollowing some commands to copy. "
                "They will highlight the elements without materials "
                "or to reset the highlighted nodes:\n"
            )
            FreeCAD.Console.PrintMessage(command_to_highlight + "\n")
            # command to reset the Highlighted Nodes
            FreeCAD.Console.PrintMessage(
                "Gui.ActiveDocument.{}.HighlightedNodes = []\n\n"
                .format(self.mesh.Name)
            )
            return True
        else:
            return False

    def has_nonpositive_jacobians(self):
        if "*ERROR in e_c3d: nonpositive jacobian" in self.ccx_stdout:
            nonpositive_jacobian_elements = []
            nonpositive_jacobian_elenodes = []
            for line in self.ccx_stdout.splitlines():
                if "determinant in element" in line:
                    # print(line)
                    # print(line.split())
                    non_posjac_ele = int(line.split()[3])
                    # print(non_posjac_ele)
                    if non_posjac_ele not in nonpositive_jacobian_elements:
                        nonpositive_jacobian_elements.append(non_posjac_ele)
            for e in nonpositive_jacobian_elements:
                for n in self.mesh.FemMesh.getElementNodes(e):
                    nonpositive_jacobian_elenodes.append(n)
            nonpositive_jacobian_elements = sorted(nonpositive_jacobian_elements)
            nonpositive_jacobian_elenodes = sorted(nonpositive_jacobian_elenodes)
            command_for_nonposjacnodes = (
                "nonpositive_jacobian_elenodes = {}"
                .format(nonpositive_jacobian_elenodes)
            )
            command_to_highlight = (
                "Gui.ActiveDocument.{}.HighlightedNodes = nonpositive_jacobian_elenodes"
                .format(self.mesh.Name)
            )
            # some output for the user
            FreeCAD.Console.PrintError(
                "\n\nCalculiX returned an error due to nonpositive jacobian elements.\n"
            )
            FreeCAD.Console.PrintMessage(
                "nonpositive_jacobian_elements = {}\n"
                .format(nonpositive_jacobian_elements)
            )
            FreeCAD.Console.PrintMessage(command_for_nonposjacnodes + "\n")
            if FreeCAD.GuiUp:
                import FreeCADGui
                # with this the list nonpositive_jacobian_elenodes
                # will be available for further user interaction
                FreeCADGui.doCommand(command_for_nonposjacnodes)
                FreeCAD.Console.PrintMessage("\n")
                FreeCADGui.doCommand(command_to_highlight)
            FreeCAD.Console.PrintMessage(
                "\nFollowing some commands to copy. "
                "They highlight the nonpositive jacobians "
                "or to reset the highlighted nodes:\n"
            )
            FreeCAD.Console.PrintMessage(command_to_highlight + "\n")
            # command to reset the Highlighted Nodes
            FreeCAD.Console.PrintMessage(
                "Gui.ActiveDocument.{}.HighlightedNodes = []\n\n"
                .format(self.mesh.Name)
            )
            return True
        else:
            return False

    def load_results(self):
        FreeCAD.Console.PrintMessage("We will load the ccx frd and dat result file.\n")
        self.results_present = False
        self.load_results_ccxfrd()
        self.load_results_ccxdat()

    def load_results_ccxfrd(self):
        """Load results of ccx calculations from .frd file.
        """
        import feminout.importCcxFrdResults as importCcxFrdResults
        frd_result_file = os.path.splitext(self.inp_file_name)[0] + ".frd"
        if os.path.isfile(frd_result_file):
            importCcxFrdResults.importFrd(frd_result_file, self.analysis, "CCX_")
            for m in self.analysis.Group:
                if m.isDerivedFrom("Fem::FemResultObject"):
                    self.results_present = True
                    break
            else:
                if self.solver.AnalysisType == "check":
                    for m in self.analysis.Group:
                        if m.isDerivedFrom("Fem::FemMeshObjectPython"):
                            # we have no result object but a mesh object
                            # this happens in NOANALYSIS mode
                            break
                else:
                    FreeCAD.Console.PrintError("FEM: No result object in active Analysis.\n")
        else:
            raise Exception("FEM: No results found at {}!".format(frd_result_file))

    def load_results_ccxdat(self):
        """Load results of ccx calculations from .dat file.
        """
        import feminout.importCcxDatResults as importCcxDatResults
        dat_result_file = os.path.splitext(self.inp_file_name)[0] + ".dat"
        if os.path.isfile(dat_result_file):
            mode_frequencies = importCcxDatResults.import_dat(dat_result_file, self.analysis)
        else:
            raise Exception("FEM: No .dat results found at {}!".format(dat_result_file))
        if mode_frequencies:
            # print(mode_frequencies)
            for m in self.analysis.Group:
                if m.isDerivedFrom("Fem::FemResultObject") and m.Eigenmode > 0:
                    for mf in mode_frequencies:
                        if m.Eigenmode == mf["eigenmode"]:
                            m.EigenmodeFrequency = mf["frequency"]


class CcxTools(FemToolsCcx):

    def __init__(self, solver=None):
        FemToolsCcx.__init__(self, None, solver)

##  @}



# ***************************************************************************
# *   Copyright (c) 2020 Werner Mayer <wmayer[at]users.sourceforge.net>     *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FEM Utilities"
__author__ = "Werner Mayer"
__url__ = "http://www.freecadweb.org"

tokens = (
    'NAME',
    'FLOAT',
    'INT',
    'PLUS',
    'MINUS',
    'TIMES',
    'DIVIDE',
    'EQUALS',
    'LPAREN',
    'RPAREN',
    'COMMENT',
)

# Tokens

t_PLUS    = r'\+'
t_MINUS   = r'-'
t_TIMES   = r'\*'
t_DIVIDE  = r'/'
t_EQUALS  = r'='
t_LPAREN  = r'\('
t_RPAREN  = r'\)'
t_NAME    = r'[a-zA-Z_][a-zA-Z0-9_]*'


def t_FLOAT(t):
    r'\d+\.(\d+)?([eE][-+]?\d+)?'
    t.value = float(t.value)
    return t


def t_INT(t):
    r'\d+'
    t.value = int(t.value)
    return t


# Ignored characters
t_ignore = " \t"


def t_COMMENT(t):
    r'\#.*'
    pass


def t_newline(t):
    r'\n+'
    t.lexer.lineno += t.value.count("\n")


def t_error(t):
    print("Illegal character '%s'" % t.value[0])
    t.lexer.skip(1)


# Build the lexer
import ply.lex as lex
lex.lex()


# Precedence rules for the arithmetic operators
precedence = (
    ('left', 'PLUS', 'MINUS'),
    ('left', 'TIMES', 'DIVIDE'),
    ('right', 'UMINUS'),
)

# dictionary of names (for storing variables)
names = {}


def p_statement_assign(p):
    'statement : NAME EQUALS expression'
    names[p[1]] = p[3]


def p_statement_expr(p):
    'statement : expression'
    print(p[1])


def p_expression_binop(p):
    '''expression : expression PLUS expression
                  | expression MINUS expression
                  | expression TIMES expression
                  | expression DIVIDE expression'''
    if p[2] == '+'  : p[0] = p[1] + p[3]
    elif p[2] == '-': p[0] = p[1] - p[3]
    elif p[2] == '*': p[0] = p[1] * p[3]
    elif p[2] == '/': p[0] = p[1] / p[3]


def p_expression_uminus(p):
    'expression : MINUS expression %prec UMINUS'
    p[0] = -p[2]


def p_expression_group(p):
    'expression : LPAREN expression RPAREN'
    p[0] = p[2]


def p_expression_float(p):
    'expression : FLOAT'
    p[0] = p[1]


def p_expression_int(p):
    'expression : INT'
    p[0] = p[1]


def p_expression_name(p):
    'expression : NAME'
    try:
        p[0] = names[p[1]]
    except LookupError:
        print("Undefined name '%s'" % p[1])
        p[0] = 0


def p_error(p):
    print("Syntax error at '%s'" % p.value)


import ply.yacc as yacc
yacc.yacc()



# ***************************************************************************
# *   Copyright (c) 2017 Markus Hovorka <m.hovorka@live.de>                 *
# *   Copyright (c) 2018 Bernd Hahnebach <bernd@bimstatik.org>              *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FreeCAD FEM base constraint ViewProvider"
__author__ = "Markus Hovorka, Bernd Hahnebach"
__url__ = "http://www.freecadweb.org"

## @package _BaseViewProvider
#  \ingroup FEM
#  \brief FreeCAD _Base ViewProvider for FEM workbench

from six import string_types

import FreeCAD
import FreeCADGui

import FemGui  # needed to display the icons in TreeView

False if FemGui.__name__ else True  # flake8, dummy FemGui usage


class ViewProxy(object):
    """Proxy View Provider for Pythons base constraint."""

    def __init__(self, vobj):
        vobj.Proxy = self

    # needs to be overwritten, if no standard icon name is used
    # see constraint body heat source as an example
    def getIcon(self):
        """after load from FCStd file, self.icon does not exist, return constant path instead"""
        # https://forum.freecadweb.org/viewtopic.php?f=18&t=44009
        if (
            hasattr(self.Object.Proxy, "Type")
            and isinstance(self.Object.Proxy.Type, string_types)
            and self.Object.Proxy.Type.startswith("Fem::")
        ):
            icon_path = "/icons/{}.svg".format(self.Object.Proxy.Type.replace("Fem::", "FEM_"))
            FreeCAD.Console.PrintLog("{} --> {}\n".format(self.Object.Name, icon_path))
            return ":/{}".format(icon_path)
        else:
            FreeCAD.Console.PrintError("No icon returned for {}\n".format(self.Object.Name))
            FreeCAD.Console.PrintMessage("{}\n".format(self.Object.Proxy.Type))
            return ""

    def attach(self, vobj):
        self.Object = vobj.Object  # used on various places, claim childreens, get icon, etc.
        # self.ViewObject = vobj  # not used ATM

    def setEdit(self, vobj, mode=0, TaskPanel=None, hide_mesh=True):
        if TaskPanel is None:
            # avoid edit mode by return False
            # https://forum.freecadweb.org/viewtopic.php?t=12139&start=10#p161062
            return False
        if hide_mesh is True:
            # hide all FEM meshes and VTK FemPost* objects
            for o in vobj.Object.Document.Objects:
                if (
                    o.isDerivedFrom("Fem::FemMeshObject")
                    or o.isDerivedFrom("Fem::FemPostPipeline")
                    or o.isDerivedFrom("Fem::FemPostClipFilter")
                    or o.isDerivedFrom("Fem::FemPostScalarClipFilter")
                    or o.isDerivedFrom("Fem::FemPostWarpVectorFilter")
                    or o.isDerivedFrom("Fem::FemPostDataAlongLineFilter")
                    or o.isDerivedFrom("Fem::FemPostDataAtPointFilter")
                    or o.isDerivedFrom("Fem::FemPostCutFilter")
                    or o.isDerivedFrom("Fem::FemPostDataAlongLineFilter")
                    or o.isDerivedFrom("Fem::FemPostPlaneFunction")
                    or o.isDerivedFrom("Fem::FemPostSphereFunction")
                ):
                    o.ViewObject.hide()
        # show task panel
        task = TaskPanel(vobj.Object)
        FreeCADGui.Control.showDialog(task)
        return True

    def unsetEdit(self, vobj, mode=0):
        FreeCADGui.Control.closeDialog()
        return True

    def doubleClicked(self, vobj):
        guidoc = FreeCADGui.getDocument(vobj.Object.Document)
        # check if another VP is in edit mode
        # https://forum.freecadweb.org/viewtopic.php?t=13077#p104702
        if not guidoc.getInEdit():
            guidoc.setEdit(vobj.Object.Name)
        else:
            from PySide.QtGui import QMessageBox
            message = "Active Task Dialog found! Please close this one before opening  a new one!"
            QMessageBox.critical(None, "Error in tree view", message)
            FreeCAD.Console.PrintError(message + "\n")
        return True

    # they are needed, see:
    # https://forum.freecadweb.org/viewtopic.php?f=18&t=44021
    # https://forum.freecadweb.org/viewtopic.php?f=18&t=44009
    def __getstate__(self):
        return None

    def __setstate__(self, state):
        return None



#***************************************************************************
#*   (c) Yorik van Havre (yorik@uncreated.net) 2014                        *
#*                                                                         *
#*   This file is part of the FreeCAD CAx development system.              *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU Lesser General Public License (LGPL)    *
#*   as published by the Free Software Foundation; either version 2 of     *
#*   the License, or (at your option) any later version.                   *
#*   for detail see the LICENCE text file.                                 *
#*                                                                         *
#*   FreeCAD is distributed in the hope that it will be useful,            *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU Lesser General Public License for more details.                   *
#*                                                                         *
#*   You should have received a copy of the GNU Library General Public     *
#*   License along with FreeCAD; if not, write to the Free Software        *
#*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
#*   USA                                                                   *
#*                                                                         *
#***************************************************************************/


'''
These are a common functions and classes for creating custom post processors.
'''

from PySide import QtCore, QtGui
import FreeCAD

FreeCADGui = None
if FreeCAD.GuiUp:
    import FreeCADGui


class GCodeHighlighter(QtGui.QSyntaxHighlighter):
    def __init__(self, parent=None):
        super(GCodeHighlighter, self).__init__(parent)

        keywordFormat = QtGui.QTextCharFormat()
        keywordFormat.setForeground(QtCore.Qt.cyan)
        keywordFormat.setFontWeight(QtGui.QFont.Bold)
        keywordPatterns = ["\\bG[0-9]+\\b",
                           "\\bM[0-9]+\\b"]

        self.highlightingRules = [(QtCore.QRegExp(pattern), keywordFormat) for pattern in keywordPatterns]

        speedFormat = QtGui.QTextCharFormat()
        speedFormat.setFontWeight(QtGui.QFont.Bold)
        speedFormat.setForeground(QtCore.Qt.green)
        self.highlightingRules.append((QtCore.QRegExp("\\bF[0-9\\.]+\\b"),speedFormat))

    def highlightBlock(self, text):
        for pattern, hlFormat in self.highlightingRules:
            expression = QtCore.QRegExp(pattern)
            index = expression.indexIn(text)
            while index >= 0:
                length = expression.matchedLength()
                self.setFormat(index, length, hlFormat)
                index = expression.indexIn(text, index + length)



class GCodeEditorDialog(QtGui.QDialog):
    def __init__(self, parent = None):
        if parent is None:
            parent = FreeCADGui.getMainWindow()
        QtGui.QDialog.__init__(self,parent)

        layout = QtGui.QVBoxLayout(self)

        # nice text editor widget for editing the gcode
        self.editor = QtGui.QTextEdit()
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setFixedPitch(True)
        font.setPointSize(10)
        self.editor.setFont(font)
        self.editor.setText("G01 X55 Y4.5 F300.0")
        layout.addWidget(self.editor)

        # OK and Cancel buttons
        self.buttons = QtGui.QDialogButtonBox(
            QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel,
            QtCore.Qt.Horizontal, self)
        layout.addWidget(self.buttons)

        # restore placement and size
        self.paramKey = "User parameter:BaseApp/Values/Mod/Path/GCodeEditor/"
        params = FreeCAD.ParamGet(self.paramKey)
        posX = params.GetInt("posX")
        posY = params.GetInt("posY")
        if posX > 0 and posY > 0:
            self.move(posX, posY)
        width = params.GetInt("width")
        height = params.GetInt("height")
        if width > 0 and height > 0:
            self.resize(width, height)

        self.buttons.accepted.connect(self.accept)
        self.buttons.rejected.connect(self.reject)

    def done(self, *args, **kwargs):
        params = FreeCAD.ParamGet(self.paramKey)
        params.SetInt("posX", self.x())
        params.SetInt("posY", self.y())
        params.SetInt("width", self.size().width())
        params.SetInt("height", self.size().height())
        return QtGui.QDialog.done(self, *args, **kwargs)


def stringsplit(commandline):
    returndict = {'command':None, 'X':None, 'Y':None, 'Z':None, 'A':None, 'B':None, 'F':None, 'T':None, 'S':None, 'I':None, 'J':None,'K':None, 'txt': None}
    wordlist = [a.strip() for a in commandline.split(" ")]
    if wordlist[0][0] == '(':
        returndict['command'] = 'message'
        returndict['txt'] = wordlist[0]
    else:
        returndict['command'] = wordlist[0]
    for word in wordlist[1:]:
        returndict[word[0]] = word[1:]

    return returndict 

def fmt(num,dec,units):
    ''' used to format axis moves, feedrate, etc for decimal places and units'''
    if units == 'G21': #metric
        fnum = '%.*f' % (dec, num)
    else: #inch
        fnum = '%.*f' % (dec, num/25.4) #since FreeCAD uses metric units internally
    return fnum

def editor(gcode):
    '''pops up a handy little editor to look at the code output '''
    
    prefs = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Path")
    # default Max Highlighter Size = 512 Ko
    defaultMHS = 512 * 1024 
    mhs = prefs.GetUnsigned('inspecteditorMaxHighlighterSize', defaultMHS)
    
    dia = GCodeEditorDialog()
    dia.editor.setText(gcode)
    gcodeSize = len(dia.editor.toPlainText())
    if (gcodeSize <= mhs):
        # because of poor performance, syntax highlighting is 
        # limited to mhs octets (default 512 KB).
        # It seems than the response time curve has an inflexion near 500 KB
        # beyond 500 KB, the response time increases exponentially.
        dia.highlighter = GCodeHighlighter(dia.editor.document())
    else:
        FreeCAD.Console.PrintMessage(translate("Path", "GCode size too big ({} o), disabling syntax highlighter.".format(gcodeSize)))
    result = dia.exec_()
    if result:  # If user selected 'OK' get modified G Code
        final = dia.editor.toPlainText()
    else:
        final = gcode
    return final

def fcoms(string,commentsym):
    ''' filter and rebuild comments with user preferred comment symbol'''
    if len(commentsym)==1:
        s1 = string.replace('(', commentsym)
        comment = s1.replace(')', '')
    else:
        return string
    return comment






#   (c) Juergen Riegel (FreeCAD@juergen-riegel.net) 2011      LGPL        *
#                                                                         *
#   This file is part of the FreeCAD CAx development system.              *
#                                                                         *
#   This program is free software; you can redistribute it and/or modify  *
#   it under the terms of the GNU Lesser General Public License (LGPL)    *
#   as published by the Free Software Foundation; either version 2 of     *
#   the License, or (at your option) any later version.                   *
#   for detail see the LICENCE text file.                                 *
#                                                                         *
#   FreeCAD is distributed in the hope that it will be useful,            *
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#   GNU Library General Public License for more details.                  *
#                                                                         *
#   You should have received a copy of the GNU Library General Public     *
#   License along with FreeCAD; if not, write to the Free Software        *
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
#   USA                                                                   *
#**************************************************************************
import unittest

import FreeCAD

App = FreeCAD

class TestDraft(unittest.TestCase):
    def setUp(self):
        self.Doc = FreeCAD.newDocument("PartDesignTestDraft")

    def testSimpleDraft(self):
        # fix: create datum plane on YZ. create datum line on Z + 10i
        # find top face by first making list comprehension of Z-normal faces
        # and then find which has the higher center of mass Z-value
        self.Body = self.Doc.addObject('PartDesign::Body','Body')
        self.Box = self.Doc.addObject('PartDesign::AdditiveBox','Box')
        self.Body.addObject(self.Box)
        self.Box.Length=10.00
        self.Box.Width=10.00
        self.Box.Height=10.00
        self.Doc.recompute()
        self.DatumPlane = self.Doc.addObject('PartDesign::Plane','DatumPlane')
        self.DatumPlane.Support = [(self.Doc.YZ_Plane,'')]
        self.DatumPlane.MapMode = 'FlatFace'
        self.Body.addObject(self.DatumPlane)
        self.Doc.recompute()
        self.DatumLine = self.Doc.addObject('PartDesign::Line','DatumLine')
        self.DatumLine.Support = [(self.Doc.X_Axis,'')]
        self.DatumLine.MapMode = 'TwoPointLine'
        self.Body.addObject(self.DatumLine)
        self.Doc.recompute()
        self.Draft = self.Doc.addObject("PartDesign::Draft","Draft")
        # Draft.Base needs to be top face
        self.Faces = self.Box.Shape.Faces
        # Grab the two faces with Z-normals and find the higher one
        self.ZFaceIndexes = [i for i in range(len(self.Faces)) if self.Faces[i].Surface.Axis == App.Vector(0,0,1)]
        if self.Faces[self.ZFaceIndexes[0]].CenterOfMass.z > self.Faces[self.ZFaceIndexes[1]].CenterOfMass.z:
            self.TopFaceIndex = self.ZFaceIndexes[0]
        else:
            self.TopFaceIndex = self.ZFaceIndexes[1]
        self.Draft.Base = (self.Box, ["Face"+str(self.TopFaceIndex+1)])
        self.Draft.NeutralPlane = (self.DatumPlane, [''])
        self.Draft.PullDirection = (self.DatumLine, [''])
        self.Draft.Angle = 45.0
        self.Draft.Reversed = 1
        self.Body.addObject(self.Draft)
        self.Doc.recompute()
        if 'Invalid' in self.Draft.State:
            self.Draft.Reversed = 0
            self.Doc.recompute()
        self.assertAlmostEqual(self.Draft.Shape.Volume, 1500)

    def tearDown(self):
        #closing doc
        FreeCAD.closeDocument("PartDesignTestDraft")
        # print ("omit closing document for debugging")




# ***************************************************************************
# *   Copyright (c) 2009, 2010 Yorik van Havre <yorik@uncreated.net>        *
# *   Copyright (c) 2009, 2010 Ken Cline <cline@frii.com>                   *
# *   Copyright (c) 2020 FreeCAD Developers                                 *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""This module provides the object code for Draft BSpline.
"""
## @package bspline
# \ingroup DRAFT
# \brief This module provides the object code for Draft BSpline.

from PySide.QtCore import QT_TRANSLATE_NOOP

import FreeCAD as App

from draftutils.utils import get_param

from draftobjects.base import DraftObject


class BSpline(DraftObject):
    """The BSpline object"""

    def __init__(self, obj):
        super(BSpline, self).__init__(obj, "BSpline")

        _tip = "The points of the B-spline"
        obj.addProperty("App::PropertyVectorList","Points",
                        "Draft", QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "If the B-spline is closed or not"
        obj.addProperty("App::PropertyBool","Closed",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "Create a face if this spline is closed"
        obj.addProperty("App::PropertyBool","MakeFace",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "The area of this object"
        obj.addProperty("App::PropertyArea","Area",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        obj.MakeFace = get_param("fillmode",True)
        obj.Closed = False
        obj.Points = []
        self.assureProperties(obj)

    def assureProperties(self, obj): # for Compatibility with older versions
        if not hasattr(obj, "Parameterization"):
            obj.addProperty("App::PropertyFloat","Parameterization","Draft",QT_TRANSLATE_NOOP("App::Property","Parameterization factor"))
            obj.Parameterization = 1.0
            self.knotSeq = []

    def parameterization (self, pts, a, closed):
        """Computes a knot Sequence for a set of points.
        fac (0-1) : parameterization factor
        fac = 0 -> Uniform / fac=0.5 -> Centripetal / fac=1.0 -> Chord-Length
        """
        if closed: # we need to add the first point as the end point
            pts.append(pts[0])
        params = [0]
        for i in range(1,len(pts)):
            p = pts[i].sub(pts[i-1])
            pl = pow(p.Length,a)
            params.append(params[-1] + pl)
        return params

    def onChanged(self, fp, prop):
        if prop == "Parameterization":
            if fp.Parameterization < 0.:
                fp.Parameterization = 0.
            if fp.Parameterization > 1.0:
                fp.Parameterization = 1.0

    def execute(self, obj):
        import Part

        self.assureProperties(obj)

        if not obj.Points:
            obj.positionBySupport()

        self.knotSeq = self.parameterization(obj.Points, obj.Parameterization, obj.Closed)
        plm = obj.Placement
        if obj.Closed and (len(obj.Points) > 2):
            if obj.Points[0] == obj.Points[-1]:  # should not occur, but OCC will crash
                _err = "_BSpline.createGeometry: \
                        Closed with same first/last Point. Geometry not updated."
                App.Console.PrintError(QT_TRANSLATE_NOOP('Draft', _err)+"\n")
                return
            spline = Part.BSplineCurve()
            spline.interpolate(obj.Points, PeriodicFlag = True, Parameters = self.knotSeq)
            # DNC: bug fix: convert to face if closed
            shape = Part.Wire(spline.toShape())
            # Creating a face from a closed spline cannot be expected to always work
            # Usually, if the spline is not flat the call of Part.Face() fails
            try:
                if hasattr(obj,"MakeFace"):
                    if obj.MakeFace:
                        shape = Part.Face(shape)
                else:
                    shape = Part.Face(shape)
            except Part.OCCError:
                pass
            obj.Shape = shape
            if hasattr(obj,"Area") and hasattr(shape,"Area"):
                obj.Area = shape.Area
        else:
            spline = Part.BSplineCurve()
            spline.interpolate(obj.Points, PeriodicFlag = False, Parameters = self.knotSeq)
            shape = spline.toShape()
            obj.Shape = shape
            if hasattr(obj,"Area") and hasattr(shape,"Area"):
                obj.Area = shape.Area
        obj.Placement = plm
        obj.positionBySupport()


_BSpline = BSpline


# ***************************************************************************
# *   Copyright (c) 2019 Bernd Hahnebach <bernd@bimstatik.org>              *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful,            *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with FreeCAD; if not, write to the Free Software        *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************


# to run the examples copy the code:
"""
from femexamples.manager import *
run_all()

from femexamples.manager import *
doc = run_boxanalysisstatic()
doc = run_boxanalysisfrequency()
doc = run_ccx_cantileverfaceload()
doc = run_ccx_cantilevernodeload()
doc = run_ccx_cantileverprescribeddisplacement()
doc = setup_cantileverhexa20faceload()
doc = run_constraint_contact_shell_shell()
doc = run_constraint_contact_solid_solid()
doc = run_constraint_tie()
doc = run_material_nl_platewithhole()
doc = run_material_multiple_twoboxes()
doc = run_rcwall2d()
doc = run_thermomech_bimetall()
doc = run_thermomech_flow1d()
doc = run_thermomech_spine()

doc = run_ccx_cantilevernodeload("calculix")
doc = run_ccx_cantilevernodeload("ccxtools")
doc = run_ccx_cantilevernodeload("z88")

"""


import FreeCAD


def run_analysis(doc, base_name, filepath=""):

    from os.path import join, exists
    from os import makedirs
    from tempfile import gettempdir as gettmp

    # recompute
    doc.recompute()

    # print(doc.Objects)
    # print([obj.Name for obj in doc.Objects])

    # filepath
    if filepath == "":
        filepath = join(gettmp(), "FEM_examples")
    if not exists(filepath):
        makedirs(filepath)

    # find solver
    # ATM we only support one solver, search for a frame work solver and run it
    for m in doc.Analysis.Group:
        from femtools.femutils import is_derived_from
        if (
            is_derived_from(m, "Fem::FemSolverObjectPython")
            and m.Proxy.Type != "Fem::FemSolverCalculixCcxTools"
        ):
            solver = m
            break

    # we need a file name for the besides dir to work
    save_fc_file = join(filepath, (base_name + ".FCStd"))
    FreeCAD.Console.PrintMessage(
        "Save FreeCAD file for {} analysis to {}\n.".format(base_name, save_fc_file)
    )
    doc.saveAs(save_fc_file)

    # get analysis workig dir
    from femtools.femutils import get_beside_dir
    working_dir = get_beside_dir(solver)

    # run analysis
    from femsolver.run import run_fem_solver
    run_fem_solver(solver, working_dir)

    # save doc once again with results
    doc.save()


def run_boxanalysisstatic(solver=None, base_name=None):

    from .boxanalysis import setup_static as setup
    doc = setup()

    if base_name is None:
        base_name = "Box_Static_Analysis"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_boxanalysisfrequency(solver=None, base_name=None):

    from .boxanalysis import setup_frequency as setup
    doc = setup()

    if base_name is None:
        base_name = "Box_Frequency_Analysis"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_ccx_cantileverfaceload(solver=None, base_name=None):

    from .ccx_cantilever_std import setup_cantileverfaceload as setup
    doc = setup()

    if base_name is None:
        base_name = "CantilverFaceLoad"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_ccx_cantilevernodeload(solver=None, base_name=None):

    from .ccx_cantilever_std import setup_cantilevernodeload as setup
    doc = setup()

    if base_name is None:
        base_name = "CantileverNodeLoad"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_ccx_cantileverprescribeddisplacement(solver=None, base_name=None):

    from .ccx_cantilever_std import setup_cantileverprescribeddisplacement as setup
    doc = setup()

    if base_name is None:
        base_name = "CantileverPrescribedDisplacement"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def setup_cantileverhexa20faceload(solver=None, base_name=None):

    from .ccx_cantilever_std import setup_cantileverhexa20faceload as setup
    doc = setup()

    if base_name is None:
        base_name = "CantilverHexa20FaceLoad"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_constraint_contact_shell_shell(solver=None, base_name=None):

    from .constraint_contact_shell_shell import setup
    doc = setup()

    if base_name is None:
        base_name = "Constraint_Contact_Shell_Shell"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_constraint_contact_solid_solid(solver=None, base_name=None):

    from .constraint_contact_solid_solid import setup
    doc = setup()

    if base_name is None:
        base_name = "Constraint_Contact_Solid_Solid"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_constraint_tie(solver=None, base_name=None):

    from .constraint_tie import setup
    doc = setup()

    if base_name is None:
        base_name = "Constraint_Tie"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_material_multiple_twoboxes(solver=None, base_name=None):

    from .material_multiple_twoboxes import setup
    doc = setup()

    if base_name is None:
        base_name = "Multimaterial_Two-Boxes"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_material_nl_platewithhole(solver=None, base_name=None):

    from .material_nl_platewithhole import setup
    doc = setup()

    if base_name is None:
        base_name = "Nonlinear_material_plate_with_hole"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_rcwall2d(solver=None, base_name=None):

    from .rc_wall_2d import setup
    doc = setup()

    if base_name is None:
        base_name = "RC_FIB_Wall_2D"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_thermomech_bimetall(solver=None, base_name=None):

    from .thermomech_bimetall import setup
    doc = setup()

    if base_name is None:
        base_name = "Thermomech_Bimetall"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_thermomech_flow1d(solver=None, base_name=None):

    from .thermomech_flow1d import setup
    doc = setup()

    if base_name is None:
        base_name = "Thermomech_Spine"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_thermomech_spine(solver=None, base_name=None):

    from .thermomech_spine import setup
    doc = setup()

    if base_name is None:
        base_name = "Thermomech_Spine"
        if solver is not None:
            base_name += "_" + solver
    run_analysis(doc, base_name)
    doc.recompute()

    return doc


def run_all():
    run_boxanalysisstatic()
    run_boxanalysisfrequency()
    run_ccx_cantileverfaceload()
    run_ccx_cantilevernodeload()
    run_ccx_cantileverprescribeddisplacement()
    run_constraint_contact_shell_shell()
    run_constraint_contact_solid_solid()
    run_material_nl_platewithhole()
    run_material_multiple_twoboxes()
    run_rcwall2d()
    run_thermomech_bimetall()
    run_thermomech_flow1d()
    run_thermomech_spine()



# -*- coding: utf-8 -*-

# ***************************************************************************
# *   Copyright (c) 2020 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

"""
Provides all gui and tools to create and edit annotation styles
Provides Draft_AnnotationStyleEditor command
"""

import FreeCAD,FreeCADGui
import json

def QT_TRANSLATE_NOOP(ctx,txt): return txt

param = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")

DEFAULT = {
    "FontName":("font",param.GetString("textfont","Sans")),
    "FontSize":("str",str(param.GetFloat("textheight",100))),
    "LineSpacing":("str","1 cm"),
    "ScaleMultiplier":("float",1),
    "ShowUnit":("bool",False),
    "UnitOverride":("str",""),
    "Decimals":("int",2),
    "ShowLines":("bool",True),
    "LineWidth":("int",param.GetInt("linewidth",1)),
    "LineColor":("color",param.GetInt("color",255)),
    "ArrowType":("index",param.GetInt("dimsymbol",0)),
    "ArrowSize":("str",str(param.GetFloat("arrowsize",20))),
    "DimensionOvershoot":("str",str(param.GetFloat("dimovershoot",20))),
    "ExtensionLines":("str",str(param.GetFloat("extlines",300))),
    "ExtensionOvershoot":("str",str(param.GetFloat("extovershoot",20))),
    }


class Draft_AnnotationStyleEditor:

    def __init__(self):

        self.styles = {}
        self.renamed = {}

    def GetResources(self):

        return {'Pixmap'  : ":icons/Draft_Annotation_Style.svg",
                'MenuText': QT_TRANSLATE_NOOP("Draft_AnnotationStyleEditor", "Annotation styles..."),
                'ToolTip' : QT_TRANSLATE_NOOP("Draft_AnnotationStyleEditor", "Manage or create annotation styles")}

    def IsActive(self):

        return bool(FreeCAD.ActiveDocument)

    def Activated(self):

        from PySide import QtGui

        # reset rename table
        self.renamed = {}

        # load dialog
        self.form = FreeCADGui.PySideUic.loadUi(":/ui/dialog_AnnotationStyleEditor.ui")

        # restore stored size
        w = param.GetInt("AnnotationStyleEditorWidth",450)
        h = param.GetInt("AnnotationStyleEditorHeight",450)
        self.form.resize(w,h)

        # center the dialog over FreeCAD window
        mw = FreeCADGui.getMainWindow()
        self.form.move(mw.frameGeometry().topLeft() + mw.rect().center() - self.form.rect().center())

        # set icons
        self.form.setWindowIcon(QtGui.QIcon(":/icons/Draft_Annotation_Style.svg"))
        self.form.pushButtonDelete.setIcon(QtGui.QIcon(":/icons/edit_Cancel.svg"))
        self.form.pushButtonRename.setIcon(QtGui.QIcon(":/icons/accessories-text-editor.svg"))

        # fill the styles combo
        self.styles = self.read_meta()
        for style in self.styles.keys():
            self.form.comboBoxStyles.addItem(style)

        # connect signals/slots
        self.form.comboBoxStyles.currentIndexChanged.connect(self.on_style_changed)
        self.form.pushButtonDelete.clicked.connect(self.on_delete)
        self.form.pushButtonRename.clicked.connect(self.on_rename)
        for attr in DEFAULT.keys():
            control = getattr(self.form,attr)
            for signal in ["clicked","textChanged","valueChanged","stateChanged","currentIndexChanged"]:
                if hasattr(control,signal):
                    getattr(control,signal).connect(self.update_style)
                    break

        # show editor dialog
        result = self.form.exec_()

        # process if OK was clicked
        if result:
            self.save_meta(self.styles)

        # store dialog size
        param.SetInt("AnnotationStyleEditorWidth",self.form.width())
        param.SetInt("AnnotationStyleEditorHeight",self.form.height())

        return

    def read_meta(self):

        """reads the document Meta property and returns a dict"""

        styles = {}
        meta = FreeCAD.ActiveDocument.Meta
        for key,value in meta.items():
            if key.startswith("Draft_Style_"):
                styles[key[12:]] = json.loads(value)
        return styles

    def save_meta(self,styles):

        """saves a dict to the document Meta property and updates objects"""

        # save meta
        changedstyles = []
        meta = FreeCAD.ActiveDocument.Meta
        for key,value in styles.items():
            try:
                strvalue = json.dumps(value)
            except:
                print("debug: unable to serialize this:",value)
            if ("Draft_Style_"+key in meta) and (meta["Draft_Style_"+key] != strvalue):
                changedstyles.append(key)
            meta["Draft_Style_"+key] = strvalue
        # remove deleted styles
        todelete = []
        for key,value in meta.items():
            if key.startswith("Draft_Style_"):
                if key[12:] not in styles:
                    todelete.append(key)
        for key in todelete:
            del meta[key]
        
        FreeCAD.ActiveDocument.Meta = meta

        # propagate changes to all annotations
        for obj in self.get_annotations():
            if obj.ViewObject.AnnotationStyle in self.renamed.keys():
                # temporarily add the new style and switch to it
                obj.ViewObject.AnnotationStyle = obj.ViewObject.AnnotationStyle+[self.renamed[obj.ViewObject.AnnotationStyle]]
                obj.ViewObject.AnnotationStyle = self.renamed[obj.ViewObject.AnnotationStyle]
            if obj.ViewObject.AnnotationStyle in styles.keys():
                if obj.ViewObject.AnnotationStyle in changedstyles:
                    for attr,attrvalue in styles[obj.ViewObject.AnnotationStyle].items():
                        if hasattr(obj.ViewObject,attr):
                            setattr(obj.ViewObject,attr,attrvalue)
            else:
                obj.ViewObject.AnnotationStyle = ""
            obj.ViewObject.AnnotationStyle == [""] + styles.keys()

    def on_style_changed(self,index):

        """called when the styles combobox is changed"""

        from PySide import QtGui

        if index <= 1:
            # nothing happens
            self.form.pushButtonDelete.setEnabled(False)
            self.form.pushButtonRename.setEnabled(False)
            self.fill_editor(None)
        if index == 1:
            # Add new... entry
            reply = QtGui.QInputDialog.getText(None, "Create new style","Style name:")
            if reply[1]:
                # OK or Enter pressed
                name = reply[0]
                if name in self.styles:
                    reply = QtGui.QMessageBox.information(None,"Style exists","This style name already exists")
                else:
                    # create new default style
                    self.styles[name] = {}
                    for key,val in DEFAULT.items():
                        self.styles[name][key] = val[1]
                    self.form.comboBoxStyles.addItem(name)
                    self.form.comboBoxStyles.setCurrentIndex(self.form.comboBoxStyles.count()-1)
        elif index > 1:
            # Existing style
            self.form.pushButtonDelete.setEnabled(True)
            self.form.pushButtonRename.setEnabled(True)
            self.fill_editor(self.form.comboBoxStyles.itemText(index))

    def on_delete(self):

        """called when the Delete button is pressed"""

        from PySide import QtGui

        index = self.form.comboBoxStyles.currentIndex()
        style = self.form.comboBoxStyles.itemText(index)
        if self.get_style_users(style):
            reply = QtGui.QMessageBox.question(None, "Style in use", "This style is used by some objects in this document. Are you sure?",
                QtGui.QMessageBox.Yes | QtGui.QMessageBox.No, QtGui.QMessageBox.No)
            if reply == QtGui.QMessageBox.No:
                return
        self.form.comboBoxStyles.removeItem(index)
        del self.styles[style]

    def on_rename(self):

        """called when the Rename button is pressed"""

        from PySide import QtGui

        index = self.form.comboBoxStyles.currentIndex()
        style = self.form.comboBoxStyles.itemText(index)
        reply = QtGui.QInputDialog.getText(None, "Rename style","New name:",QtGui.QLineEdit.Normal,style)
        if reply[1]:
            # OK or Enter pressed
            newname = reply[0]
            if newname in self.styles:
                reply = QtGui.QMessageBox.information(None,"Style exists","This style name already exists")
            else:
                self.form.comboBoxStyles.setItemText(index,newname)
                value = self.styles[style]
                del self.styles[style]
                self.styles[newname] = value
                self.renamed[style] = newname

    def fill_editor(self,style):

        """fills the editor fields with the contents of a style"""

        from PySide import QtGui

        if style is None:
            style = {}
            for key,val in DEFAULT.items():
                style[key] = val[1]
        if not isinstance(style,dict):
            if style in self.styles:
                style = self.styles[style]
            else:
                print("debug: unable to fill dialog from style",style)
        for key,value in style.items():
            control = getattr(self.form,key)
            if DEFAULT[key][0] == "str":
                control.setText(value)
            elif DEFAULT[key][0] == "font":
                control.setCurrentFont(QtGui.QFont(value))
            elif DEFAULT[key][0] == "color":
                r = ((value>>24)&0xFF)/255.0
                g = ((value>>16)&0xFF)/255.0
                b = ((value>>8)&0xFF)/255.0
                color = QtGui.QColor.fromRgbF(r,g,b)
                control.setProperty("color",color)
            elif DEFAULT[key][0] in ["int","float"]:
                control.setValue(value)
            elif DEFAULT[key][0] == "bool":
                control.setChecked(value)
            elif DEFAULT[key][0] == "index":
                control.setCurrentIndex(value)

    def update_style(self,arg=None):

        """updates the current style with the values from the editor"""

        index = self.form.comboBoxStyles.currentIndex()
        if index > 1:
            values = {}
            style = self.form.comboBoxStyles.itemText(index)
            for key in DEFAULT.keys():
                control = getattr(self.form,key)
                if DEFAULT[key][0] == "str":
                    values[key] = control.text()
                elif DEFAULT[key][0] == "font":
                    values[key] = control.currentFont().family()
                elif DEFAULT[key][0] == "color":
                    values[key] = control.property("color").rgb()<<8
                elif DEFAULT[key][0] in ["int","float"]:
                    values[key] = control.value()
                elif DEFAULT[key][0] == "bool":
                    values[key] = control.isChecked()
                elif DEFAULT[key][0] == "index":
                    values[key] = control.currentIndex()
            self.styles[style] = values

    def get_annotations(self):

        """gets all the objects that support annotation styles"""

        users = []
        for obj in FreeCAD.ActiveDocument.Objects:
            vobj = obj.ViewObject
            if hasattr(vobj,"AnnotationStyle"):
                users.append(obj)
        return users

    def get_style_users(self,style):

        """get all objects using a certain style"""

        users = []
        for obj in self.get_annotations():
            if obj.ViewObject.AnnotationStyle == style:
                users.append(obj)
        return users


FreeCADGui.addCommand('Draft_AnnotationStyleEditor', Draft_AnnotationStyleEditor())



# -*- coding: utf-8 -*-

# Resource object code
#
# Created: Sun May 3 20:14:20 2020
#      by: The Resource Compiler for PySide2 (Qt v5.12.5)
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore

qt_resource_data = b"\
\x00\x00\x17]\
<\
?xml version=\x221.\
0\x22 encoding=\x22UTF\
-8\x22 standalone=\x22\
no\x22?>\x0d\x0a<!-- Crea\
ted with Inkscap\
e (http://www.in\
kscape.org/) -->\
\x0d\x0a\x0d\x0a<svg\x0d\x0a   xml\
ns:dc=\x22http://pu\
rl.org/dc/elemen\
ts/1.1/\x22\x0d\x0a   xml\
ns:cc=\x22http://cr\
eativecommons.or\
g/ns#\x22\x0d\x0a   xmlns\
:rdf=\x22http://www\
.w3.org/1999/02/\
22-rdf-syntax-ns\
#\x22\x0d\x0a   xmlns:svg\
=\x22http://www.w3.\
org/2000/svg\x22\x0d\x0a \
  xmlns=\x22http://\
www.w3.org/2000/\
svg\x22\x0d\x0a   xmlns:x\
link=\x22http://www\
.w3.org/1999/xli\
nk\x22\x0d\x0a   xmlns:so\
dipodi=\x22http://s\
odipodi.sourcefo\
rge.net/DTD/sodi\
podi-0.dtd\x22\x0d\x0a   \
xmlns:inkscape=\x22\
http://www.inksc\
ape.org/namespac\
es/inkscape\x22\x0d\x0a  \
 width=\x2264\x22\x0d\x0a   \
height=\x2264\x22\x0d\x0a   \
id=\x22svg2\x22\x0d\x0a   ve\
rsion=\x221.1\x22\x0d\x0a   \
inkscape:version\
=\x220.48.5 r10040\x22\
\x0d\x0a   sodipodi:do\
cname=\x22preview-v\
ector.svg\x22>\x0d\x0a  <\
defs\x0d\x0a     id=\x22d\
efs4\x22>\x0d\x0a    <ink\
scape:perspectiv\
e\x0d\x0a       sodipo\
di:type=\x22inkscap\
e:persp3d\x22\x0d\x0a    \
   inkscape:vp_x\
=\x22-206.25075 : 1\
19.68854 : 1\x22\x0d\x0a \
      inkscape:v\
p_y=\x220 : 1000 : \
0\x22\x0d\x0a       inksc\
ape:vp_z=\x22183.50\
001 : 140.32143 \
: 1\x22\x0d\x0a       ink\
scape:persp3d-or\
igin=\x2248 : 32 : \
1\x22\x0d\x0a       id=\x22p\
erspective3792\x22 \
/>\x0d\x0a    <linearG\
radient\x0d\x0a       \
id=\x22linearGradie\
nt3755\x22>\x0d\x0a      \
<stop\x0d\x0a         \
style=\x22stop-colo\
r:#424242;stop-o\
pacity:1;\x22\x0d\x0a    \
     offset=\x220\x22\x0d\
\x0a         id=\x22st\
op3757\x22 />\x0d\x0a    \
  <stop\x0d\x0a       \
  style=\x22stop-co\
lor:#9f9f9f;stop\
-opacity:1;\x22\x0d\x0a  \
       offset=\x221\
\x22\x0d\x0a         id=\x22\
stop3759\x22 />\x0d\x0a  \
  </linearGradie\
nt>\x0d\x0a    <linear\
Gradient\x0d\x0a      \
 inkscape:collec\
t=\x22always\x22\x0d\x0a    \
   xlink:href=\x22#\
linearGradient37\
55\x22\x0d\x0a       id=\x22\
linearGradient37\
61\x22\x0d\x0a       x1=\x22\
75.724594\x22\x0d\x0a    \
   y1=\x2285.023956\
\x22\x0d\x0a       x2=\x2241\
.895164\x22\x0d\x0a      \
 y2=\x2213.334851\x22\x0d\
\x0a       gradient\
Units=\x22userSpace\
OnUse\x22 />\x0d\x0a  </d\
efs>\x0d\x0a  <sodipod\
i:namedview\x0d\x0a   \
  id=\x22base\x22\x0d\x0a   \
  pagecolor=\x22#ff\
ffff\x22\x0d\x0a     bord\
ercolor=\x22#666666\
\x22\x0d\x0a     borderop\
acity=\x221.0\x22\x0d\x0a   \
  inkscape:pageo\
pacity=\x220.0\x22\x0d\x0a  \
   inkscape:page\
shadow=\x222\x22\x0d\x0a    \
 inkscape:zoom=\x22\
8.7571168\x22\x0d\x0a    \
 inkscape:cx=\x2239\
.859065\x22\x0d\x0a     i\
nkscape:cy=\x2230.9\
25664\x22\x0d\x0a     ink\
scape:document-u\
nits=\x22px\x22\x0d\x0a     \
inkscape:current\
-layer=\x22g3794\x22\x0d\x0a\
     showgrid=\x22t\
rue\x22\x0d\x0a     showb\
order=\x22true\x22\x0d\x0a  \
   inkscape:wind\
ow-width=\x22800\x22\x0d\x0a\
     inkscape:wi\
ndow-height=\x22836\
\x22\x0d\x0a     inkscape\
:window-x=\x220\x22\x0d\x0a \
    inkscape:win\
dow-y=\x2227\x22\x0d\x0a    \
 inkscape:window\
-maximized=\x220\x22\x0d\x0a\
     inkscape:sn\
ap-bbox=\x22false\x22\x0d\
\x0a     inkscape:s\
nap-nodes=\x22true\x22\
\x0d\x0a     inkscape:\
snap-global=\x22tru\
e\x22>\x0d\x0a    <inksca\
pe:grid\x0d\x0a       \
type=\x22xygrid\x22\x0d\x0a \
      id=\x22grid30\
02\x22\x0d\x0a       emps\
pacing=\x222\x22\x0d\x0a    \
   visible=\x22true\
\x22\x0d\x0a       enable\
d=\x22true\x22\x0d\x0a      \
 snapvisiblegrid\
linesonly=\x22true\x22\
 />\x0d\x0a  </sodipod\
i:namedview>\x0d\x0a  \
<metadata\x0d\x0a     \
id=\x22metadata7\x22>\x0d\
\x0a    <rdf:RDF>\x0d\x0a\
      <cc:Work\x0d\x0a\
         rdf:abo\
ut=\x22\x22>\x0d\x0a        \
<dc:format>image\
/svg+xml</dc:for\
mat>\x0d\x0a        <d\
c:type\x0d\x0a        \
   rdf:resource=\
\x22http://purl.org\
/dc/dcmitype/Sti\
llImage\x22 />\x0d\x0a   \
     <dc:title /\
>\x0d\x0a        <dc:c\
reator>\x0d\x0a       \
   <cc:Agent>\x0d\x0a \
           <dc:t\
itle>$committer<\
/dc:title>\x0d\x0a    \
      </cc:Agent\
>\x0d\x0a        </dc:\
creator>\x0d\x0a      \
  <dc:title>prev\
iew-vector</dc:t\
itle>\x0d\x0a        <\
dc:date>2013-11-\
19</dc:date>\x0d\x0a  \
      <dc:relati\
on>http://www.fr\
eecadweb.org/wik\
i/index.php?titl\
e=Artwork</dc:re\
lation>\x0d\x0a       \
 <dc:publisher>\x0d\
\x0a          <cc:A\
gent>\x0d\x0a         \
   <dc:title>Fre\
eCAD</dc:title>\x0d\
\x0a          </cc:\
Agent>\x0d\x0a        \
</dc:publisher>\x0d\
\x0a        <dc:ide\
ntifier>FreeCAD/\
src/Mod/Material\
/Resources/icons\
/preview-vector.\
svg</dc:identifi\
er>\x0d\x0a        <dc\
:rights>\x0d\x0a      \
    <cc:Agent>\x0d\x0a\
            <dc:\
title>FreeCAD LG\
PL2+</dc:title>\x0d\
\x0a          </cc:\
Agent>\x0d\x0a        \
</dc:rights>\x0d\x0a  \
      <cc:licens\
e>https://www.gn\
u.org/copyleft/l\
esser.html</cc:l\
icense>\x0d\x0a       \
 <dc:contributor\
>\x0d\x0a          <cc\
:Agent>\x0d\x0a       \
     <dc:title>[\
agryson] Alexand\
er Gryson</dc:ti\
tle>\x0d\x0a          \
</cc:Agent>\x0d\x0a   \
     </dc:contri\
butor>\x0d\x0a      </\
cc:Work>\x0d\x0a    </\
rdf:RDF>\x0d\x0a  </me\
tadata>\x0d\x0a  <g\x0d\x0a \
    inkscape:lab\
el=\x22Layer 1\x22\x0d\x0a  \
   inkscape:grou\
pmode=\x22layer\x22\x0d\x0a \
    id=\x22layer1\x22\x0d\
\x0a     transform=\
\x22translate(0,-98\
8.36218)\x22>\x0d\x0a    \
<g\x0d\x0a       id=\x22g\
3794\x22\x0d\x0a       st\
yle=\x22color:#0000\
00;fill:#ffffff;\
fill-opacity:1;f\
ill-rule:evenodd\
;stroke:none;str\
oke-width:3.2939\
9991;marker:none\
;visibility:visi\
ble;display:inli\
ne;overflow:visi\
ble;enable-backg\
round:accumulate\
\x22>\x0d\x0a      <path\x0d\
\x0a         style=\
\x22fill:#d3d7cf;fi\
ll-opacity:1;fil\
l-rule:evenodd;s\
troke:#000000;st\
roke-width:2;str\
oke-linecap:roun\
d;stroke-linejoi\
n:round;stroke-m\
iterlimit:4;stro\
ke-opacity:1;str\
oke-dasharray:no\
ne\x22\x0d\x0a         d=\
\x22m 29.814326,991\
.36218 -8.814326\
,4 -1,7.00002 26\
,1 14.994091,-8.\
6577 z\x22\x0d\x0a       \
  id=\x22path3816\x22\x0d\
\x0a         inksca\
pe:connector-cur\
vature=\x220\x22\x0d\x0a    \
     sodipodi:no\
detypes=\x22cccccc\x22\
 />\x0d\x0a      <path\
\x0d\x0a         d=\x22M \
38.484871,1007.8\
171 40,1046.3622\
 l 21,-22.4044 0\
,-29.25647 z\x22\x0d\x0a \
        style=\x22f\
ill:#b7b7b7;fill\
-opacity:1;fill-\
rule:evenodd;str\
oke:#000000;stro\
ke-width:2;strok\
e-linecap:round;\
stroke-linejoin:\
round;stroke-mit\
erlimit:4;stroke\
-opacity:1;strok\
e-dasharray:none\
\x22\x0d\x0a         id=\x22\
path3818\x22\x0d\x0a     \
    inkscape:con\
nector-curvature\
=\x220\x22\x0d\x0a         s\
odipodi:nodetype\
s=\x22ccccc\x22 />\x0d\x0a  \
    <path\x0d\x0a     \
    style=\x22fill:\
#babdb6;stroke:#\
d3d7cf;stroke-wi\
dth:2;stroke-lin\
ecap:butt;stroke\
-linejoin:miter;\
stroke-opacity:1\
\x22\x0d\x0a         d=\x22M\
 59.035745,9.662\
7807 59.108006,3\
4.818961 40.9153\
1,54.264958 42.6\
98526,19.372964 \
z\x22\x0d\x0a         id=\
\x22path3022\x22\x0d\x0a    \
     inkscape:co\
nnector-curvatur\
e=\x220\x22\x0d\x0a         \
transform=\x22trans\
late(0,988.36218\
)\x22\x0d\x0a         sod\
ipodi:nodetypes=\
\x22ccccc\x22 />\x0d\x0a    \
  <path\x0d\x0a       \
  d=\x22m 21,995.36\
218 31,5.00002 -\
12,46 -30,-8 z\x22\x0d\
\x0a         style=\
\x22fill:#ffffff;fi\
ll-opacity:1;fil\
l-rule:evenodd;s\
troke:#000000;st\
roke-width:4;str\
oke-linejoin:rou\
nd;stroke-miterl\
imit:4;stroke-op\
acity:1;stroke-d\
asharray:none\x22\x0d\x0a\
         id=\x22pat\
h3820\x22\x0d\x0a        \
 inkscape:connec\
tor-curvature=\x220\
\x22\x0d\x0a         sodi\
podi:nodetypes=\x22\
ccccc\x22 />\x0d\x0a     \
 <path\x0d\x0a        \
 d=\x22m 21,995.362\
18 31,5.00002 -1\
2,46 -30,-8 z\x22\x0d\x0a\
         style=\x22\
color:#000000;fi\
ll:none;stroke:#\
888a85;stroke-wi\
dth:2;stroke-lin\
ejoin:round;stro\
ke-miterlimit:4;\
stroke-opacity:1\
;stroke-dasharra\
y:none;marker:no\
ne;visibility:vi\
sible;display:in\
line;overflow:vi\
sible;enable-bac\
kground:accumula\
te\x22\x0d\x0a         id\
=\x22path3820-3\x22\x0d\x0a \
        inkscape\
:connector-curva\
ture=\x220\x22\x0d\x0a      \
   sodipodi:node\
types=\x22ccccc\x22 />\
\x0d\x0a    </g>\x0d\x0a  </\
g>\x0d\x0a</svg>\x0d\x0a\
\x00\x00\x18\xe0\
<\
?xml version=\x221.\
0\x22 encoding=\x22UTF\
-8\x22 standalone=\x22\
no\x22?>\x0d\x0a<!-- Crea\
ted with Inkscap\
e (http://www.in\
kscape.org/) -->\
\x0d\x0a<svg xmlns:dc=\
\x22http://purl.org\
/dc/elements/1.1\
/\x22 xmlns:cc=\x22htt\
p://creativecomm\
ons.org/ns#\x22 xml\
ns:rdf=\x22http://w\
ww.w3.org/1999/0\
2/22-rdf-syntax-\
ns#\x22 xmlns:svg=\x22\
http://www.w3.or\
g/2000/svg\x22 xmln\
s=\x22http://www.w3\
.org/2000/svg\x22 x\
mlns:xlink=\x22http\
://www.w3.org/19\
99/xlink\x22 xmlns:\
sodipodi=\x22http:/\
/sodipodi.source\
forge.net/DTD/so\
dipodi-0.dtd\x22 xm\
lns:inkscape=\x22ht\
tp://www.inkscap\
e.org/namespaces\
/inkscape\x22 width\
=\x2264\x22 height=\x2264\
\x22 id=\x22svg2\x22 vers\
ion=\x221.1\x22 inksca\
pe:version=\x220.48\
.5 r10040\x22 sodip\
odi:docname=\x22pre\
view-rendered.sv\
g\x22>\x0d\x0a  <defs id=\
\x22defs4\x22>\x0d\x0a    <l\
inearGradient id\
=\x22linearGradient\
3755\x22>\x0d\x0a      <s\
top style=\x22stop-\
color:#424242;st\
op-opacity:1;\x22 o\
ffset=\x220\x22 id=\x22st\
op3757\x22/>\x0d\x0a     \
 <stop style=\x22st\
op-color:#9f9f9f\
;stop-opacity:1;\
\x22 offset=\x221\x22 id=\
\x22stop3759\x22/>\x0d\x0a  \
  </linearGradie\
nt>\x0d\x0a    <filter\
 color-interpola\
tion-filters=\x22sR\
GB\x22 inkscape:col\
lect=\x22always\x22 id\
=\x22filter3871\x22 x=\
\x22-0.45363569\x22 wi\
dth=\x221.9072714\x22 \
y=\x22-0.45363569\x22 \
height=\x221.907271\
4\x22>\x0d\x0a      <feGa\
ussianBlur inksc\
ape:collect=\x22alw\
ays\x22 stdDeviatio\
n=\x229.8975056\x22 id\
=\x22feGaussianBlur\
3873\x22/>\x0d\x0a    </f\
ilter>\x0d\x0a    <lin\
earGradient inks\
cape:collect=\x22al\
ways\x22 xlink:href\
=\x22#linearGradien\
t3600\x22 id=\x22linea\
rGradient3606\x22 x\
1=\x2251.037281\x22 y1\
=\x2247.692612\x22 x2=\
\x2214.872466\x22 y2=\x22\
10.397644\x22 gradi\
entUnits=\x22userSp\
aceOnUse\x22/>\x0d\x0a   \
 <linearGradient\
 id=\x22linearGradi\
ent3600\x22>\x0d\x0a     \
 <stop style=\x22st\
op-color:#555753\
;stop-opacity:1\x22\
 offset=\x220\x22 id=\x22\
stop3602\x22/>\x0d\x0a   \
   <stop style=\x22\
stop-color:#d3d7\
cf;stop-opacity:\
1\x22 offset=\x221\x22 id\
=\x22stop3604\x22/>\x0d\x0a \
   </linearGradi\
ent>\x0d\x0a    <filte\
r inkscape:colle\
ct=\x22always\x22 id=\x22\
filter3805\x22 x=\x22-\
0.29648924\x22 widt\
h=\x221.5929785\x22 y=\
\x22-0.30309633\x22 he\
ight=\x221.6061927\x22\
>\x0d\x0a      <feGaus\
sianBlur inkscap\
e:collect=\x22alway\
s\x22 stdDeviation=\
\x222.2490136\x22 id=\x22\
feGaussianBlur38\
07\x22/>\x0d\x0a    </fil\
ter>\x0d\x0a  </defs>\x0d\
\x0a  <sodipodi:nam\
edview id=\x22base\x22\
 pagecolor=\x22#fff\
fff\x22 bordercolor\
=\x22#666666\x22 borde\
ropacity=\x221.0\x22 i\
nkscape:pageopac\
ity=\x220.0\x22 inksca\
pe:pageshadow=\x222\
\x22 inkscape:zoom=\
\x224.8193618\x22 inks\
cape:cx=\x22-4.0027\
486\x22 inkscape:cy\
=\x2228.204234\x22 ink\
scape:document-u\
nits=\x22px\x22 inksca\
pe:current-layer\
=\x22layer1\x22 showgr\
id=\x22true\x22 showbo\
rder=\x22true\x22 inks\
cape:window-widt\
h=\x22800\x22 inkscape\
:window-height=\x22\
837\x22 inkscape:wi\
ndow-x=\x220\x22 inksc\
ape:window-y=\x2227\
\x22 inkscape:windo\
w-maximized=\x220\x22>\
\x0d\x0a    <inkscape:\
grid type=\x22xygri\
d\x22 id=\x22grid2988\x22\
 empspacing=\x222\x22 \
visible=\x22true\x22 e\
nabled=\x22true\x22 sn\
apvisiblegridlin\
esonly=\x22true\x22/>\x0d\
\x0a  </sodipodi:na\
medview>\x0d\x0a  <met\
adata id=\x22metada\
ta7\x22>\x0d\x0a    <rdf:\
RDF>\x0d\x0a      <cc:\
Work rdf:about=\x22\
\x22>\x0d\x0a        <dc:\
format>image/svg\
+xml</dc:format>\
\x0d\x0a        <dc:ty\
pe rdf:resource=\
\x22http://purl.org\
/dc/dcmitype/Sti\
llImage\x22/>\x0d\x0a    \
    <dc:title/>\x0d\
\x0a        <dc:cre\
ator>\x0d\x0a         \
 <cc:Agent>\x0d\x0a   \
         <dc:tit\
le>$committer</d\
c:title>\x0d\x0a      \
    </cc:Agent>\x0d\
\x0a        </dc:cr\
eator>\x0d\x0a        \
<dc:title>previe\
w-rendered</dc:t\
itle>\x0d\x0a        <\
dc:date>2013-11-\
19</dc:date>\x0d\x0a  \
      <dc:relati\
on>http://www.fr\
eecadweb.org/wik\
i/index.php?titl\
e=Artwork</dc:re\
lation>\x0d\x0a       \
 <dc:publisher>\x0d\
\x0a          <cc:A\
gent>\x0d\x0a         \
   <dc:title>Fre\
eCAD</dc:title>\x0d\
\x0a          </cc:\
Agent>\x0d\x0a        \
</dc:publisher>\x0d\
\x0a        <dc:ide\
ntifier>FreeCAD/\
src/Mod/Material\
/Resources/icons\
/preview-rendere\
d.svg</dc:identi\
fier>\x0d\x0a        <\
dc:rights>\x0d\x0a    \
      <cc:Agent>\
\x0d\x0a            <d\
c:title>FreeCAD \
LGPL2+</dc:title\
>\x0d\x0a          </c\
c:Agent>\x0d\x0a      \
  </dc:rights>\x0d\x0a\
        <cc:lice\
nse>https://www.\
gnu.org/copyleft\
/lesser.html</cc\
:license>\x0d\x0a     \
   <dc:contribut\
or>\x0d\x0a          <\
cc:Agent>\x0d\x0a     \
       <dc:title\
>[agryson] Alexa\
nder Gryson</dc:\
title>\x0d\x0a        \
  </cc:Agent>\x0d\x0a \
       </dc:cont\
ributor>\x0d\x0a      \
</cc:Work>\x0d\x0a    \
</rdf:RDF>\x0d\x0a  </\
metadata>\x0d\x0a  <g \
inkscape:label=\x22\
Layer 1\x22 inkscap\
e:groupmode=\x22lay\
er\x22 id=\x22layer1\x22 \
transform=\x22trans\
late(0,-988.3621\
8)\x22>\x0d\x0a    <path \
sodipodi:type=\x22a\
rc\x22 style=\x22color\
:#000000;fill:#0\
00000;fill-opaci\
ty:0.65625;fill-\
rule:evenodd;str\
oke:none;stroke-\
width:5;marker:n\
one;visibility:v\
isible;display:i\
nline;overflow:v\
isible;filter:ur\
l(#filter3871);e\
nable-background\
:accumulate\x22 id=\
\x22path2826-8\x22 sod\
ipodi:cx=\x2231.636\
364\x22 sodipodi:cy\
=\x2230.363636\x22 sod\
ipodi:rx=\x2226.181\
818\x22 sodipodi:ry\
=\x2226.181818\x22 d=\x22\
m 57.818182,30.3\
63636 a 26.18181\
8,26.181818 0 1 \
1 -52.363636,0 2\
6.181818,26.1818\
18 0 1 1 52.3636\
36,0 z\x22 transfor\
m=\x22matrix(0.4806\
1681,0,0,0.12015\
42,24.795032,104\
2.7139)\x22/>\x0d\x0a    \
<path sodipodi:t\
ype=\x22arc\x22 style=\
\x22color:#000000;f\
ill:url(#linearG\
radient3606);fil\
l-opacity:1;fill\
-rule:evenodd;st\
roke:#0b1521;str\
oke-width:1.8056\
4272;marker:none\
;visibility:visi\
ble;display:inli\
ne;overflow:visi\
ble;enable-backg\
round:accumulate\
\x22 id=\x22path2826\x22 \
sodipodi:cx=\x2231.\
636364\x22 sodipodi\
:cy=\x2230.363636\x22 \
sodipodi:rx=\x2226.\
181818\x22 sodipodi\
:ry=\x2226.181818\x22 \
d=\x22m 57.818182,3\
0.363636 a 26.18\
1818,26.181818 0\
 1 1 -52.363636,\
0 26.181818,26.1\
81818 0 1 1 52.3\
63636,0 z\x22 trans\
form=\x22matrix(1.1\
076389,0,0,1.107\
6389,-3.0416672,\
986.73024)\x22/>\x0d\x0a \
   <path sodipod\
i:type=\x22arc\x22 sty\
le=\x22color:#00000\
0;fill:none;stro\
ke:#d3d7cf;strok\
e-width:1.939394\
12000000000;mark\
er:none;visibili\
ty:visible;displ\
ay:inline;overfl\
ow:visible;enabl\
e-background:acc\
umulate\x22 id=\x22pat\
h2826-4\x22 sodipod\
i:cx=\x2231.636364\x22\
 sodipodi:cy=\x2230\
.363636\x22 sodipod\
i:rx=\x2226.181818\x22\
 sodipodi:ry=\x2226\
.181818\x22 d=\x22m 57\
.818182,30.36363\
6 a 26.181818,26\
.181818 0 1 1 -5\
2.363636,0 26.18\
1818,26.181818 0\
 1 1 52.363636,0\
 z\x22 transform=\x22m\
atrix(1.03125,0,\
0,1.03125,-0.625\
00046,989.04968)\
\x22/>\x0d\x0a    <path s\
odipodi:type=\x22ar\
c\x22 style=\x22opacit\
y:0.844827569999\
99986;color:#000\
000;fill:#ffffff\
;fill-opacity:1;\
fill-rule:evenod\
d;stroke:none;st\
roke-width:5;mar\
ker:none;visibil\
ity:visible;disp\
lay:inline;overf\
low:visible;enab\
le-background:ac\
cumulate;filter:\
url(#filter3805)\
\x22 id=\x22path3608\x22 \
sodipodi:cx=\x2211.\
181818\x22 sodipodi\
:cy=\x2220.181818\x22 \
sodipodi:rx=\x2210.\
454545\x22 sodipodi\
:ry=\x227.2727275\x22 \
d=\x22m 21.636363,2\
0.181818 a 10.45\
4545,7.2727275 0\
 1 1 -20.9090900\
1,0 10.454545,7.\
2727275 0 1 1 20\
.90909001,0 z\x22 t\
ransform=\x22matrix\
(0.85867478,-0.7\
467709,0.7057652\
6,0.90856459,-4.\
8393203,997.4188\
)\x22/>\x0d\x0a    <path \
sodipodi:type=\x22a\
rc\x22 style=\x22color\
:#000000;fill:no\
ne;stroke:#2e343\
6;stroke-width:1\
.805642720000000\
03;marker:none;v\
isibility:visibl\
e;display:inline\
;overflow:visibl\
e;enable-backgro\
und:accumulate\x22 \
id=\x22path2826-5\x22 \
sodipodi:cx=\x2231.\
636364\x22 sodipodi\
:cy=\x2230.363636\x22 \
sodipodi:rx=\x2226.\
181818\x22 sodipodi\
:ry=\x2226.181818\x22 \
d=\x22m 57.818182,3\
0.363636 a 26.18\
1818,26.181818 0\
 1 1 -52.363636,\
0 26.181818,26.1\
81818 0 1 1 52.3\
63636,0 z\x22 trans\
form=\x22matrix(1.1\
076389,0,0,1.107\
6389,-3.0416672,\
986.73024)\x22/>\x0d\x0a \
 </g>\x0d\x0a</svg>\x0d\x0a\
"

qt_resource_name = b"\
\x00\x05\
\x00o\xa6S\
\x00i\
\x00c\x00o\x00n\x00s\
\x00\x12\
\x0eO\xfeG\
\x00p\
\x00r\x00e\x00v\x00i\x00e\x00w\x00-\x00v\x00e\x00c\x00t\x00o\x00r\x00.\x00s\x00v\
\x00g\
\x00\x14\
\x02\x9c\x05\x07\
\x00p\
\x00r\x00e\x00v\x00i\x00e\x00w\x00-\x00r\x00e\x00n\x00d\x00e\x00r\x00e\x00d\x00.\
\x00s\x00v\x00g\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00\x02\
\x00\x00\x00:\x00\x00\x00\x00\x00\x01\x00\x00\x17a\
\x00\x00\x00\x10\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
